/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.annotation.processor.generator;

import java.io.InputStream;
import java.io.StringWriter;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.is2t.giml.description.Description;
import com.is2t.giml.description.DescriptionFactory;
import com.is2t.giml.description.DescriptionPackage;
import com.is2t.giml.description.Type;
import com.is2t.giml.description.Type.Basetype;

/**
 * The class names interface exposes fully qualified names of some classes, as strings.
 */
public interface ClassName {

	/*
	 * Standard classes
	 */
	/**
	 * The {@link ArrayList} class fully qualified name.
	 */
	String ARRAYLIST = ArrayList.class.getCanonicalName();
	/**
	 * The {@link Collection} class fully qualified name.
	 */
	String COLLECTION = Collection.class.getCanonicalName();
	/**
	 * The {@link Collections} class fully qualified name.
	 */
	String COLLECTIONS = Collections.class.getCanonicalName();
	/**
	 * The {@link IllegalArgumentException} class fully qualified name.
	 */
	String ILLEGALARGUMENTEXCEPTION = IllegalArgumentException.class.getCanonicalName();
	/**
	 * The {@link List} class fully qualified name.
	 */
	String LIST = List.class.getCanonicalName();
	/**
	 * The {@link Map} class fully qualified name.
	 */
	String MAP = Map.class.getCanonicalName();
	/**
	 * The {@link Map.Entry} class fully qualified name.
	 */
	String MAP_ENTRY = Map.Entry.class.getCanonicalName();
	/**
	 * The {@link Set} class fully qualified name.
	 */
	String SET = Set.class.getCanonicalName();
	/**
	 * The {@link Object} class fully qualified name.
	 */
	String OBJECT = Object.class.getCanonicalName();
	/**
	 * The {@link String} class fully qualified name.
	 */
	String STRING = String.class.getCanonicalName();
	/**
	 * The {@link StringWriter} class fully qualified name.
	 */
	String STRINGWRITER = StringWriter.class.getCanonicalName();
	/**
	 * The {@link InputStream} class fully qualified name.
	 */
	String INPUTSTREAM = InputStream.class.getCanonicalName();
	/**
	 * The {@link ClassLoader} class fully qualified name.
	 */
	String CLASSLOADER = ClassLoader.class.getCanonicalName();
	/**
	 * The {@link Class} class fully qualified name.
	 */
	String CLASS = Class.class.getCanonicalName();
	/**
	 * The {@link Constructor} class fully qualified name.
	 */
	String CONSTRUCTOR = Constructor.class.getCanonicalName();

	/**
	 * The {@link Exception} class fully qualified name.
	 */
	String EXCEPTION = Exception.class.getCanonicalName();
	/**
	 * The {@link ReflectiveOperationException} class fully qualified name.
	 */
	String REFLECTIVEOPERATIONEXCEPTION = ReflectiveOperationException.class.getCanonicalName();

	/*
	 * GIML classes
	 */
	/**
	 * The {@link Basetype} class fully qualified name.
	 */
	String GIML_BASETYPE = Basetype.class.getCanonicalName().replace('$', '.');
	/**
	 * The {@link Description} class fully qualified name.
	 */
	String GIML_DESCRIPTION = Description.class.getCanonicalName();
	/**
	 * The {@link DescriptionFactory} class fully qualified name.
	 */
	String GIML_DESCRIPTIONFACTORY = DescriptionFactory.class.getCanonicalName();
	/**
	 * The {@link DescriptionPackage} class fully qualified name.
	 */
	String GIML_DESCRIPTIONPACKAGE = DescriptionPackage.class.getCanonicalName();
	/**
	 * The {@link Type} class fully qualified name.
	 */
	String GIML_TYPE = Type.class.getCanonicalName();

}
