/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.cssgenerator.selector;

import java.io.StringWriter;
import java.util.Hashtable;
import java.util.Map;

import org.w3c.css.sac.Condition;
import org.w3c.css.sac.DescendantSelector;

import com.steadystate.css.parser.selectors.DescendantSelectorImpl;

/**
 *
 */
public class DescendantCombinator implements Selector {

	private static final String DESCENDANT_COMBINATOR_SELECTOR_VAR_PREFIX = "descendantCombi_";
	private final Map<String, String> descendantCombinatorSelectorVars = new Hashtable<String, String>();

	@Override
	public String generate(Condition c, StringWriter writer) throws IllegalArgumentException {
		throw new RuntimeException();
	}

	@Override
	public String generate(org.w3c.css.sac.Selector s, StringWriter writer) throws IllegalArgumentException {
		DescendantSelector descendantSelector = this.get(s);

		String ancestorSelectorVar = SelectorFactory.create(descendantSelector.getAncestorSelector(), writer);
		String simpleSelectorVar = SelectorFactory.create(descendantSelector.getSimpleSelector(), writer);
		StringBuilder descendantCombinatorSelector = new StringBuilder(ancestorSelectorVar).append(simpleSelectorVar);
		String descendantSelectorVar = this.descendantCombinatorSelectorVars
				.get(descendantCombinatorSelector.toString());
		if (descendantSelectorVar != null) {
			return descendantSelectorVar;
		}
		StringBuilder descendantSelectorVarSb = new StringBuilder(DESCENDANT_COMBINATOR_SELECTOR_VAR_PREFIX)
				.append(ancestorSelectorVar).append("_").append(simpleSelectorVar);
		writer.append(this.getClassName()).append(" ").append(descendantSelectorVarSb);
		writer.append(" = new ");
		writer.append(this.getClassName()).append('(');
		writer.append(ancestorSelectorVar);
		writer.append(", ");
		writer.append(simpleSelectorVar);
		writer.append(");\n");
		this.descendantCombinatorSelectorVars.put(descendantCombinatorSelector.toString(),
				descendantSelectorVarSb.toString());
		return descendantSelectorVarSb.toString();
	}

	private DescendantSelector get(org.w3c.css.sac.Selector s) {
		if (!this.matches(s)) {
			throw new IllegalArgumentException("Given selector is not a DescendantSelector");
		}

		return (DescendantSelector) s;
	}

	@Override
	public boolean matches(Condition condition) {
		return false;
	}

	@Override
	public String getClassName() {
		return "ej.style.selector.combinator.DescendantCombinator";
	}

	@Override
	public boolean matches(org.w3c.css.sac.Selector selector) {
		return selector instanceof DescendantSelectorImpl;
	}

}
