/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.jsgenerator;

import java.io.IOException;
import java.io.Reader;
import java.util.List;

import jdk.nashorn.internal.ir.Block;
import jdk.nashorn.internal.ir.FunctionNode;
import jdk.nashorn.internal.ir.Statement;
import jdk.nashorn.internal.parser.Parser;
import jdk.nashorn.internal.runtime.Context;
import jdk.nashorn.internal.runtime.ErrorManager;
import jdk.nashorn.internal.runtime.Source;
import jdk.nashorn.internal.runtime.options.Options;

/**
 * A JavaScript parser for inlined code based on Nashorn Parser The Nashorn
 * parser has been choosen as a first parser since it it already available in
 * JRE 8, without any external dependencies, and seems to be sufficient for the
 * GIML JS language scope.
 */
public class JavaScriptParser {

	private final Options options;

	public JavaScriptParser() {
		options = new Options("nashorn");
		options.set("anon.functions", true);
		options.set("parse.only", true);
		options.set("scripting", true);
	}

	/**
	 * @return null on syntax error
	 * @throws IOException
	 */
	public List<Statement> parseInlinedCode(Reader reader) throws IOException {
		ErrorManager errors = new ErrorManager();
		Context context = new Context(options, errors, Thread.currentThread().getContextClassLoader());
		Source source = Source.sourceFor("test", reader);
		Parser parser = new Parser(context.getEnv(), source, errors);
		FunctionNode functionNode = parser.parse();
		if (errors.hasErrors()) {
			return null;
		}
		Block block = functionNode.getBody();
		List<Statement> statements = block.getStatements();
		return statements;
	}
}
