/*
 * Java
 *
 * Copyright 2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.jsgenerator.test;

import org.junit.Test;

/**
 *
 */
public class TestExpressions extends AbstractTest {

	@Test
	public void testNullLiteral() {
		testJSSourceToMicroEJSource("null");
	}

	@Test
	public void testStringLiteral() {
		testJSSourceToMicroEJSource("'aString'", "\"aString\"");
	}

	@Test
	public void testIntegerLiteralPositive() {
		testJSSourceToMicroEJSource("2");
		testJSSourceToMicroEJSource("2147483647"); // Integer.MAX_VALUE;
	}

	@Test
	public void testLongLiteralPositive() {
		testJSSourceToMicroEJSource("2147483648", "2147483648l"); // Integer.MAX_VALUE+1;
		testJSSourceToMicroEJSource("9223372036854775807", "9223372036854775807l"); // Long.MAX_VALUE;
	}

	@Test
	public void testDoubleLiteral() {
		// JavaScript Numbers are Always 64-bit Floating Point
		testJSSourceToMicroEJSource("3.14", "3.14d");
	}

	@Test
	public void testUnaryIntegerLiteralPositiveNegative() {
		// no negative literals => parsed as unary expressions in javascript
		testJSSourceToMicroEJSource("-2");
		testJSSourceToMicroEJSource("-2147483647");
		testJSSourceToMicroEJSource("-2147483648");
		testJSSourceToMicroEJSource("+2", "2");
	}

	@Test
	public void testUnaryLongLiteralPositiveNegative() {
		// no negative literals => parsed as unary expressions in javascript
		testJSSourceToMicroEJSource("+2147483648", "2147483648l");
		testJSSourceToMicroEJSource("+9223372036854775807", "9223372036854775807l");
		testJSSourceToMicroEJSource("-9223372036854775807", "-9223372036854775807l");
		testJSSourceToMicroEJSource("-9223372036854775808", "-9223372036854775808l");
	}

	@Test
	public void testUnaryDoubleLiteralPositiveNegative() {
		testJSSourceToMicroEJSource("+3.14", "3.14d");
		testJSSourceToMicroEJSource("-3.14", "-3.14d");
	}

	@Test
	public void testBinaryOperators() {
		testJSSourceToMicroEJSource("2+2");
		testJSSourceToMicroEJSource("2&&2");
		testJSSourceToMicroEJSource("2&2");
		testJSSourceToMicroEJSource("2|2");
		testJSSourceToMicroEJSource("2^2");
		testJSSourceToMicroEJSource("2/2");
		testJSSourceToMicroEJSource("2==2");
		testJSSourceToMicroEJSource("2>=2");
		testJSSourceToMicroEJSource("2>2");
		testJSSourceToMicroEJSource("2<=2");
		testJSSourceToMicroEJSource("2<2");
		testJSSourceToMicroEJSource("2%2");
		testJSSourceToMicroEJSource("2*2");
		testJSSourceToMicroEJSource("2!=2");
		testJSSourceToMicroEJSource("2||2");
		testJSSourceToMicroEJSource("2>>>2");
		testJSSourceToMicroEJSource("2>>2");
		testJSSourceToMicroEJSource("2<<2");
		testJSSourceToMicroEJSource("2-2");
	}

	@Test
	public void testUnaryOperators() {
		testJSSourceToMicroEJSource("~2");
		testJSSourceToMicroEJSource("!2");
	}

	@Test
	public void testPrefixPostfixIncrDecr() {
		// ++ / --
		testJSSourceToMicroEJSource("++x");
		testJSSourceToMicroEJSource("--x");
		testJSSourceToMicroEJSource("++x");
		testJSSourceToMicroEJSource("--x");
	}

	@Test
	public void testFunctionCall() {
		testJSSourceToMicroEJSource("foo()");
		testJSSourceToMicroEJSource("foo(1)");
		testJSSourceToMicroEJSource("foo(--x)");
		testJSSourceToMicroEJSource("foo((20*19)+(19+82))");
	}

	@Test
	public void testComplexFunctionCallSystemOutPrintln() {
		testJSSourceToMicroEJSource("System.out.println(\"GOTO=>\"+url)");
	}

	@Test
	public void testGlobalObjectAccess() {
		testJSSourceToMicroEJSource("console.log('Hello')", "System.out.println(\"Hello\")");
	}

}
