/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.runtime.reader;

import ej.giml.GIMLReader;
import ej.nls.NLS;

/**
 *
 */
abstract public class AbstractGIMLReader implements GIMLReader {

	protected static final int FLAG_IS_NLS_STRING = 1;

	/** Pointer at the beginning of the data section */
	protected int dataPtr; // Native known

	/** Pointer at the beginning of the constant pool section */
	protected int constantPoolPtr; // Native known

	/** GIML binary id */
	private long binaryId;

	protected NLS nls;

	@Override
	public void initialize(NLS nls) {
		this.nls = nls;
		readHeader();
	}

	protected void readHeader() {
		this.binaryId = readLong();
		this.constantPoolPtr = readUInt16() + startAddress();
		this.dataPtr = readUInt16() + startAddress();
	}

	@Override
	public String readString() {
		int value = readVarUInt();
		if ((value & FLAG_IS_NLS_STRING) == 0) {
			// Not NLS string
			int relativeStringAddr = (value >> 1);
			int absoluteStringAddr = this.constantPoolPtr + relativeStringAddr;
			return decodeString(absoluteStringAddr);
		} else {
			// NLS string
			if (this.nls == null) {
				throw new NullPointerException("nls");
			}
			return this.nls.getMessage(value >> 1);
		}
	}

	@Override
	public long getBinaryId() {
		return this.binaryId;
	}

	/** Read unsigned int on 16-bits big-endian. */
	abstract protected int readUInt16();

	/** Read signed integer on 64-bits big-endian. */
	abstract protected long readLong();

	/** Decode the string encoded at the given absolute address */
	abstract protected String decodeString(int stringAddr);

	/** Returns the absolute start address used to compute data and constant pool addresses. */
	abstract protected int startAddress();

}
