/*
 * Java
 *
 * Copyright 2016-2017 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.runtime.reader;

import java.io.IOException;

import com.is2t.memory.MemoryHelper;
import com.is2t.memory.StringDecoder;

import ej.giml.GIMLReader;

/**
 *
 */
public class GIMLReaderNative extends AbstractGIMLReader {

	/** Current pointer */
	protected int ptr; // Native known

	/** Pointer at the beginning of the object map section */
	protected int objectMapPtr; // Native known

	protected int startPtr;

	/**
	 * Creates a {@link GIMLReader} that will load its data from the GIML binary referenced by the given path.
	 *
	 * @param gimlBinaryPath
	 *            absolute path to the GIML binary resource
	 * @throws IOException
	 *             on error
	 */
	public GIMLReaderNative(String gimlBinaryPath) throws IOException {
		this.startPtr = MemoryHelper.getAddress(gimlBinaryPath);
	}

	@Override
	protected void readHeader() {
		this.ptr = this.startPtr;
		super.readHeader();
		// just after the header
		this.objectMapPtr = this.ptr;
		this.ptr = this.dataPtr;
	}

	@Override
	protected int startAddress() {
		return this.startPtr;
	}

	@Override
	native public int readVarSInt();

	@Override
	native public int readVarUInt();

	@Override
	native public long readVarLong();

	@Override
	native public boolean readBoolean();

	@Override
	native public float readFloat();

	@Override
	native public double readDouble();

	@Override
	native public long readLong();

	@Override
	public void seek(int positionIdentifier) throws IllegalArgumentException {
		boolean found = seek(positionIdentifier, positionIdentifier);
		if (!found) {
			throw new IllegalArgumentException("Unknown ID: " + positionIdentifier);
		}
	}

	/**
	 * Set the position pointer on the position identified by the given identifier.
	 *
	 * @param positionIdentifier
	 *            the identifier of the position.
	 * @param hashcode
	 *            the hashcode of the positionIdentifier
	 *
	 * @return true if the position has been found, false otherwise.
	 */
	native private boolean seek(int positionIdentifier, int hashcode);

	@Override
	native protected int readUInt16();

	@Override
	protected String decodeString(int stringAddr) {
		return StringDecoder.decodeString(stringAddr);
	}

}
