/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.giml.xml.generator.test.xmlgeneratoroptions;

import org.junit.Assert;
import org.junit.Test;

import com.is2t.giml.xml.generator.XMLGeneratorConstants;
import com.is2t.giml.xml.generator.XMLGeneratorOptions;

/**
 * Unit tests for {@link XMLGeneratorOptions} class.
 */
public class XMLGeneratorOptionsTest {

	private static final String NEW_EXTENSION_1 = "ext";
	private static final String NEW_EXTENSION_2 = ".ext";
	private static final String NEW_DOCUMENT_ID = "new-document-id";
	private static final String NEW_IMPORT_TAG = "new-import-tag";
	private static final String NEW_IMPORT_TAG_ATTRIBUTE_NAME = "new-import-tag-attr-name";
	private static final String NEW_TEMPLATE_TAG = "new-template-tag";
	private static final String NEW_TEMPLATE_TAG_ATTRIBUTE_NAME = "new-template-tag-attr-name";
	private static final String NEW_TEMPLATE_PARAMETER_PREFIX = "[[prefix[[";
	private static final String NEW_TEMPLATE_PARAMETER_SUFFIX = "]]suffix]]";
	private static final String NEW_TEMPLATE_CONTENT_TAG = "new-content";
	private static final String NEW_TEMPLATE_CONTENT_TAG_ATTRIBUTE_NAME = "new-content-attr";
	private static final String NEW_TEMPLATE_CONTENT_TAG_INSTANTIATION_ATTRIBUTE_NAME = "new-content-attr-i";

	@Test
	public void testDefaultValues() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		Assert.assertEquals(options.getXMLFileExtension(), XMLGeneratorConstants.DEFAULT_XML_FILE_EXTENSION);
		Assert.assertEquals(options.getDocumentIdAttributeName(),
				XMLGeneratorConstants.DEFAULT_DOCUMENT_ID_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getImportTagName(), XMLGeneratorConstants.DEFAULT_IMPORT_DOCUMENT_TAGNAME);
		Assert.assertEquals(options.getImportTagAttributeName(),
				XMLGeneratorConstants.DEFAULT_IMPORT_NAME_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateTagName(), XMLGeneratorConstants.DEFAULT_TEMPLATE_TAGNAME);
		Assert.assertEquals(options.getTemplateTagAttributeName(),
				XMLGeneratorConstants.DEFAULT_TEMPLATE_NAME_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateParameterPrefix(),
				XMLGeneratorConstants.DEFAULT_TEMPLATE_PARAMETER_PREFIX);
		Assert.assertEquals(options.getTemplateParameterSuffix(),
				XMLGeneratorConstants.DEFAULT_TEMPLATE_PARAMETER_SUFFIX);
		Assert.assertEquals(options.getTemplateContentTagName(),
				XMLGeneratorConstants.DEFAULT_TEMPLATE_CONTENT_TAGNAME);
		Assert.assertEquals(options.getTemplateContentTagAttributeName(),
				XMLGeneratorConstants.DEFAULT_TEMPLATE_CONTENT_NAME_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateContentTagInstantiationAttributeName(),
				XMLGeneratorConstants.DEFAULT_TEMPLATE_CONTENT_INSTANTIATION_NAME_ATTRIBUTE_NAME);
	}

	@Test
	public void testEquals() {
		XMLGeneratorOptions xmlParserOptions1 = new XMLGeneratorOptions();
		XMLGeneratorOptions xmlParserOptions2 = new XMLGeneratorOptions();
		XMLGeneratorOptions xmlParserOptions3 = new XMLGeneratorOptions();
		xmlParserOptions3.setXMLFileExtension("other");

		Assert.assertFalse(xmlParserOptions1.equals(null));
		Assert.assertFalse(xmlParserOptions1.equals(new String()));
		Assert.assertFalse(xmlParserOptions1.equals(xmlParserOptions3));
		Assert.assertFalse(xmlParserOptions2.equals(xmlParserOptions3));

		Assert.assertTrue(xmlParserOptions1.equals(xmlParserOptions2));
	}

	/* Test Extension */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueXMLExtension() {
		new XMLGeneratorOptions().setXMLFileExtension(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueXMLExtension() {
		new XMLGeneratorOptions().setXMLFileExtension("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesXMLExtension() {
		new XMLGeneratorOptions().setXMLFileExtension(".ext t");
	}

	@Test
	public void setValidValueXMLExtension() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setXMLFileExtension(NEW_EXTENSION_1);
		Assert.assertEquals(options.getXMLFileExtension(), NEW_EXTENSION_1);
		options.setXMLFileExtension(NEW_EXTENSION_2);
		Assert.assertEquals(options.getXMLFileExtension(), NEW_EXTENSION_2);
	}

	/* Test Document ID attribute */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueDocumentId() {
		new XMLGeneratorOptions().setDocumentIdAttributeName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueDocumentId() {
		new XMLGeneratorOptions().setDocumentIdAttributeName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesDocumentId() {
		new XMLGeneratorOptions().setDocumentIdAttributeName("with space");
	}

	@Test
	public void setValidValueDocumentId() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setDocumentIdAttributeName(NEW_DOCUMENT_ID);
		Assert.assertEquals(options.getDocumentIdAttributeName(), NEW_DOCUMENT_ID);
	}

	/* Test Import Tag name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueImportTag() {
		new XMLGeneratorOptions().setImportTagName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueImportTag() {
		new XMLGeneratorOptions().setImportTagName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesImportTag() {
		new XMLGeneratorOptions().setImportTagName("with space");
	}

	@Test
	public void setValidValueImportTag() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setImportTagName(NEW_IMPORT_TAG);
		Assert.assertEquals(options.getImportTagName(), NEW_IMPORT_TAG);
	}

	/* Test Import Attribute Name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueImportAttributeName() {
		new XMLGeneratorOptions().setImportTagAttributeName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueImportAttributeName() {
		new XMLGeneratorOptions().setImportTagAttributeName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesImportAttributeName() {
		new XMLGeneratorOptions().setImportTagAttributeName("with space");
	}

	@Test
	public void setValidValueImportAttributeName() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setImportTagAttributeName(NEW_IMPORT_TAG_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getImportTagAttributeName(), NEW_IMPORT_TAG_ATTRIBUTE_NAME);
	}

	/* Test Template Tag name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateTag() {
		new XMLGeneratorOptions().setTemplateTagName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateTag() {
		new XMLGeneratorOptions().setTemplateTagName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateTag() {
		new XMLGeneratorOptions().setTemplateTagName("with space");
	}

	@Test
	public void setValidValueTemplateTag() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setTemplateTagName(NEW_TEMPLATE_TAG);
		Assert.assertEquals(options.getTemplateTagName(), NEW_TEMPLATE_TAG);
	}

	/* Test Template Attribute Name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateAttributeName() {
		new XMLGeneratorOptions().setTemplateTagAttributeName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateAttributeName() {
		new XMLGeneratorOptions().setTemplateTagAttributeName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateAttributeName() {
		new XMLGeneratorOptions().setTemplateTagAttributeName("with space");
	}

	@Test
	public void setValidValueTemplateAttributeName() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setTemplateTagAttributeName(NEW_TEMPLATE_TAG_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateTagAttributeName(), NEW_TEMPLATE_TAG_ATTRIBUTE_NAME);
	}

	/* Test Template parameter prefix */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateParameterPrefix() {
		new XMLGeneratorOptions().setTemplateParameterPrefix(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateParameterPrefix() {
		new XMLGeneratorOptions().setTemplateParameterPrefix("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateParameterPrefix() {
		new XMLGeneratorOptions().setTemplateParameterPrefix("with space");
	}

	@Test
	public void setValidValueTemplateParameterPrefix() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setTemplateParameterPrefix(NEW_TEMPLATE_PARAMETER_PREFIX);
		Assert.assertEquals(options.getTemplateParameterPrefix(), NEW_TEMPLATE_PARAMETER_PREFIX);
	}

	/* Test Template parameter suffix */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateParameterSuffix() {
		new XMLGeneratorOptions().setTemplateParameterSuffix(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateParameterSuffix() {
		new XMLGeneratorOptions().setTemplateParameterSuffix("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateParameterSuffix() {
		new XMLGeneratorOptions().setTemplateParameterSuffix("with space");
	}

	@Test
	public void setValidValueTemplateParameterSuffix() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setTemplateParameterSuffix(NEW_TEMPLATE_PARAMETER_SUFFIX);
		Assert.assertEquals(options.getTemplateParameterSuffix(), NEW_TEMPLATE_PARAMETER_SUFFIX);
	}

	/* Test Template content tag name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateContentTag() {
		new XMLGeneratorOptions().setTemplateContentTagName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateContentTag() {
		new XMLGeneratorOptions().setTemplateContentTagName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateContentTag() {
		new XMLGeneratorOptions().setTemplateContentTagName("with space");
	}

	@Test
	public void setValidValueTemplateContentTag() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setTemplateContentTagName(NEW_TEMPLATE_CONTENT_TAG);
		Assert.assertEquals(options.getTemplateContentTagName(), NEW_TEMPLATE_CONTENT_TAG);
	}

	/* Test Template content tag attribute name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateContentTagAttributeName() {
		new XMLGeneratorOptions().setTemplateContentTagAttributeName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateContentTagAttributeName() {
		new XMLGeneratorOptions().setTemplateContentTagAttributeName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateContentTagAttributeName() {
		new XMLGeneratorOptions().setTemplateContentTagAttributeName("with space");
	}

	@Test
	public void setValidValueTemplateContentTagAttributeName() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setTemplateContentTagAttributeName(NEW_TEMPLATE_CONTENT_TAG_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateContentTagAttributeName(), NEW_TEMPLATE_CONTENT_TAG_ATTRIBUTE_NAME);
	}

	/* Test Template content tag attribute name */

	@Test(expected = IllegalArgumentException.class)
	public void setNullValueTemplateContentTagInstantiationAttributeName() {
		new XMLGeneratorOptions().setTemplateContentTagInstantiationAttributeName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void setEmptyValueTemplateContentTagInstantiationAttributeName() {
		new XMLGeneratorOptions().setTemplateContentTagInstantiationAttributeName("");
	}

	@Test(expected = IllegalArgumentException.class)
	public void setValueWithSpacesTemplateContentTagInstantiationAttributeName() {
		new XMLGeneratorOptions().setTemplateContentTagInstantiationAttributeName("with space");
	}

	@Test
	public void setValidValueTemplateContentTagInstantiationAttributeName() {
		XMLGeneratorOptions options = new XMLGeneratorOptions();
		options.setTemplateContentTagInstantiationAttributeName(NEW_TEMPLATE_CONTENT_TAG_INSTANTIATION_ATTRIBUTE_NAME);
		Assert.assertEquals(options.getTemplateContentTagInstantiationAttributeName(),
				NEW_TEMPLATE_CONTENT_TAG_INSTANTIATION_ATTRIBUTE_NAME);
	}

}
