package com.is2t.java.nodes.expressions;

/**
 * Java
 *
 * Copyright 2008-2011 IS2T. All rights reserved
 * IS2T PROPRIETARY/ Use is subject to license terms.
 */
import com.is2t.java.nodes.Expression;
import com.is2t.java.nodes.TypeReference;

/**
 * Java Expression for object or array allocation
 * 		see subclasses
 */
public abstract class Allocation extends Expression {
	
	// ICETEA kind of allocation
	public static final int DefaultAllocation = 0;
	public static final int New    = 0;
	public static final int Alloca = 1;
	public static final int Map	   = 2;
	/**
	 * Inline allocation on stack.
	 */
	public static final int Inline = 3;

	private static final int ALLOCATION_KIND_MASK = 0x3;
	private static final int ALLOCATION_KIND_SHIFT = 0;
	private static final int NOINIT_KIND_MASK = 0x1;
	private static final int NOINIT_KIND_SHIFT = 2;

	private int allocationFlags;
	
	// object type
	public TypeReference type;
	
	// parameters  : expression for dimensions for Array, Arguments for Class
	//	parameters.length() == array dimension
	//   parameters[i] is null when size of ième dimension is unknown
	//	 For example for int[2][] -> parameters[0] = 2, parameters[1] = null
	public Expression[] parameters;		//can be null for array if initial value are used. 

	public Allocation( TypeReference type, Expression[] parameters){
		this.type = type;
		this.parameters = parameters;
	}

	public void setAllocationKind(int kind) {
		allocationFlags = (allocationFlags&~(ALLOCATION_KIND_MASK<<ALLOCATION_KIND_SHIFT)) | (kind << ALLOCATION_KIND_SHIFT);
	}

	public int getAllocationKind() {
		return (allocationFlags>>>ALLOCATION_KIND_SHIFT) & ALLOCATION_KIND_MASK;
	}

	public void setNoInit() {
		allocationFlags |= 1 << NOINIT_KIND_SHIFT;
	}

	public boolean isNoInit() {
		return (allocationFlags & (NOINIT_KIND_SHIFT<<NOINIT_KIND_MASK)) != 0;
	}
}
