/*
 * Java
 *
 * Copyright 2016 IS2T. All rights reserved.
 * IS2T PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package com.is2t.nls.generator.test;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.junit.Assert;
import org.junit.Ignore;
import org.junit.Test;

import com.is2t.nls.StringEncoder;
import com.is2t.nls.generator.NLSGenerator;
import com.is2t.nls.generator.NLSGeneratorException;
import com.is2t.nls.generator.NLSLocale;

public class TestNLSGenerator {

	private static final String CINQ = "cinq";
	private static final String DEUX = "deux";
	private static final String UN = "un";
	private static final String BONJOUR_LE_MONDE = "Bonjour le monde";
	private static final String UN_OISEAU = "un oiseau";
	private static final String CINCO = "cinco";
	private static final String DOS = "dos";
	private static final String UNO = "uno";
	private static final String HOLA_MUNDO = "Hole mundo";
	private static final String UN_PAJARO = "un pajaro";
	private static final String ONE = "one";
	private static final String TWO = "two";
	private static final String HELLO_WORLD = "hello_world";
	private static final String A_BIRD = "a_bird";
	private static final String FIVE = "five";

	@Test
	public void test() throws NLSGeneratorException, IOException {

		Set<NLSLocale> locales = new HashSet<NLSLocale>();
		NLSLocale localeFr = createFrLocale();
		locales.add(localeFr);
		locales.add(localeFr);
		locales.add(createFrLocale());
		localeFr = createFrLocale();
		localeFr.setLocaleName("fr2");
		locales.add(localeFr);
		localeFr = createFrLocale();
		localeFr.setLocaleName("fr3");
		locales.add(localeFr);
		localeFr = createFrLocale();
		localeFr.setLocaleName("fr4");
		locales.add(localeFr);
		locales.add(createFrLocale());
		locales.add(createEsLocale());

		Map<String, Integer> msgKeyIds = new HashMap<String, Integer>();
		msgKeyIds.put(HELLO_WORLD, new Integer(3));
		msgKeyIds.put(A_BIRD, new Integer(4));
		msgKeyIds.put(FIVE, new Integer(5));
		msgKeyIds.put(TWO, new Integer(2));
		msgKeyIds.put(ONE, new Integer(1));

		File outPutDir = Helper.createTmpDir();
		NLSGenerator nlsGenerator = new NLSGenerator(locales, msgKeyIds, outPutDir, new StringEncoding(), "testnls",
				"com/is2t/nls");
		nlsGenerator.generate();

		File generatedNlsBinaryFile = new File(outPutDir, "com/is2t/nls/testnls.bin");
		Assert.assertTrue(generatedNlsBinaryFile.exists());
		Assert.assertTrue(generatedNlsBinaryFile.isFile());

		byte[] binaryData = Helper.getInputStreamByteArray(new FileInputStream(generatedNlsBinaryFile));
		Assert.assertTrue(binaryData != null);

		// expected array
		ByteArrayOutputStream expectedOut = new ByteArrayOutputStream();
		expectedOut.write("es".getBytes());
		expectedOut.write("spanish".getBytes());
		expectedOut.write(UNO.getBytes());
		expectedOut.write(DOS.getBytes());
		expectedOut.write(HOLA_MUNDO.getBytes());
		expectedOut.write(UN_PAJARO.getBytes());
		expectedOut.write(CINCO.getBytes());
		expectedOut.write("fr".getBytes());
		expectedOut.write("french".getBytes());
		expectedOut.write(UN.getBytes());
		expectedOut.write(DEUX.getBytes());
		expectedOut.write(BONJOUR_LE_MONDE.getBytes());
		expectedOut.write(UN_OISEAU.getBytes());
		expectedOut.write(CINQ.getBytes());
		expectedOut.write("fr2".getBytes());
		expectedOut.write("fr3".getBytes());
		expectedOut.write("fr4".getBytes());
		Assert.assertArrayEquals(expectedOut.toByteArray(), binaryData);

	}

	private NLSLocale createFrLocale() {
		NLSLocale locale = new NLSLocale("fr", "french");
		locale.addTranslation(ONE, UN);
		locale.addTranslation(TWO, DEUX);
		locale.addTranslation(HELLO_WORLD, BONJOUR_LE_MONDE);
		locale.addTranslation(A_BIRD, UN_OISEAU);
		locale.addTranslation(FIVE, CINQ);
		return locale;
	}

	private NLSLocale createEsLocale() {
		NLSLocale locale = new NLSLocale("es", "spanish");
		locale.addTranslation(ONE, UNO);
		locale.addTranslation(TWO, DOS);
		locale.addTranslation(HELLO_WORLD, HOLA_MUNDO);
		locale.addTranslation(A_BIRD, UN_PAJARO);
		locale.addTranslation(FIVE, CINCO);
		return locale;
	}

	@Ignore
	class StringDecoder {

		String decodeString(byte[] bytes, int offset, int length) {
			return new String(bytes, offset, length);
		}
	}

	@Ignore
	public class StringEncoding extends StringEncoder {

		@Override
		public byte[] encodeString(String string) {
			return string.getBytes();
		}

		@Override
		public String decodeString(byte[] bytes) {
			return new String(bytes);
		}

	}
}
