/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.drawing;

import ej.drawing.ShapePainter;
import ej.drawing.ShapePainter.Cap;
import ej.drawing.TransformPainter.Flip;
import ej.microui.MicroUIException;
import ej.microui.display.GraphicsContext;
import ej.microui.display.Image;

/**
 * LLGRAPHICS_EXT_impl.h
 */
public class DrawingPainterNatives {

	/*
	 * Native error constants
	 *
	 * WARNING copied in microui (class NSystemDisplay)
	 */

	/**
	 * no error
	 */
	public static final int NO_ERROR = 0;

	/**
	 * Native side cannot decode the file
	 */
	public static final int INVALID_TYPE = -1;

	/**
	 * oom
	 */
	public static final int OUT_OF_MEMORY = -2;

	/**
	 * File content is invalid
	 */
	public static final int INVALID_FILE = -3;

	/**
	 * File not found
	 */
	public static final int INVALID_PATH = -4;

	/**
	 * File has been closed
	 */
	public static final int FILE_CLOSED = -6;

	/**
	 * Anonymous error
	 */
	public static final int NOK = -9;

	private DrawingPainterNatives() {
		// static class
	}

	/**
	 * LLGRAPHICS_EXT_IMPL_drawAAPoint
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param thickness
	 *            {@link AntiAliasedShape#getThickness()}
	 * @param fade
	 *            {@link AntiAliasedShape#getFade()}
	 * @param x
	 *            the x coordinate of the point
	 * @param y
	 *            the y coordinate of the point
	 */
	public static native void drawThickFadedPoint(byte[] gc, int x, int y, int thickness, int fade);

	/**
	 * LLGRAPHICS_EXT_IMPL_drawAALine
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param thickness
	 *            {@link AntiAliasedShape#getThickness()}
	 * @param fade
	 *            {@link AntiAliasedShape#getFade()}
	 * @param x1
	 *            the x coordinate of the start of the line
	 * @param y1
	 *            the y coordinate of the start of the line
	 * @param x2
	 *            the x coordinate of the end of the line
	 * @param y2
	 *            the y coordinate of the end of the line
	 * @param capStart
	 *            see {@link AntiAliasedShape#setStartCap(Cap)}
	 * @param capEnd
	 *            see {@link AntiAliasedShape#setEndCap(Cap)}
	 */
	public static native void drawThickFadedLine(byte[] gce, int x1, int y1, int x2, int y2, int thickness, int fade,
			int capStart, int capEnd);

	/**
	 * LLGRAPHICS_EXT_IMPL_drawAACircleArc
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param thickness
	 *            {@link AntiAliasedShape#getThickness()}
	 * @param fade
	 *            {@link AntiAliasedShape#getFade()}
	 * @param x
	 *            the x coordinate of the upper-left corner of the rectangle where the arc is drawn
	 * @param y
	 *            the y coordinate of the upper-left corner of the rectangle where the arc is drawn
	 * @param diameter
	 *            the diameter of the arc to draw
	 * @param startAngle
	 *            the beginning angle of the arc to draw
	 * @param arcAngle
	 *            the angular extent of the arc from <code>startAngle</code>
	 * @param capStart
	 *            see {@link AntiAliasedShape#setStartCap(Cap)}
	 * @param capEnd
	 *            see {@link AntiAliasedShape#setEndCap(Cap)}
	 */
	public static native void drawThickFadedCircleArc(byte[] gc, int x, int y, int diameter, float startAngle,
			float arcAngle, int thickness, int fade, int capStart, int capEnd);

	/**
	 * LLGRAPHICS_EXT_IMPL_drawAAEllipse
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param thickness
	 *            {@link AntiAliasedShape#getThickness()}
	 * @param fade
	 *            {@link AntiAliasedShape#getFade()}
	 * @param x
	 *            the x coordinate of the upper-left corner of the rectangle where the ellipse is drawn
	 * @param y
	 *            the y coordinate of the upper-left corner of the rectangle where the ellipse is drawn
	 * @param width
	 *            the width of the ellipse to draw
	 * @param height
	 *            the height of the ellipse to draw
	 */
	public static native void drawThickFadedEllipse(byte[] gc, int x, int y, int width, int height, int thickness,
			int fade);

	/**
	 * LLGRAPHICS_EXT_IMPL_drawAACircle
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param thickness
	 *            {@link AntiAliasedShape#getThickness()}
	 * @param fade
	 *            {@link AntiAliasedShape#getFade()}
	 * @param x
	 *            the x coordinate of the upper-left corner of the rectangle where the circle is drawn
	 * @param y
	 *            the y coordinate of the upper-left corner of the rectangle where the circle is drawn
	 * @param diameter
	 *            the diameter of the circle to draw
	 */
	public static native void drawThickFadedCircle(byte[] gc, int x, int y, int diameter, int thickness, int fade);

	/**
	 * LLGRAPHICS_EXT_IMPL_drawThickLine
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param thickness
	 *            {@link ShapePainter#getThickness()}
	 * @param x1
	 *            the x coordinate of the start of the line.
	 * @param y1
	 *            the y coordinate of the start of the line.
	 * @param x2
	 *            the x coordinate of the end of the line.
	 * @param y2
	 *            the y coordinate of the end of the line.
	 */
	public static native void drawThickLine(byte[] gc, int x1, int y1, int x2, int y2, int thickness);

	/**
	 * LLGRAPHICS_EXT_IMPL_drawThickCircleArc
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param thickness
	 *            {@link ShapePainter#getThickness()}
	 * @param x
	 *            the x coordinate of the upper-left corner of the rectangle where the arc is drawn.
	 * @param y
	 *            the y coordinate of the upper-left corner of the rectangle where the arc is drawn.
	 * @param diameter
	 *            the diameter of the arc to draw.
	 * @param startAngle
	 *            the beginning angle of the arc to draw.
	 * @param arcAngle
	 *            the angular extent of the arc from <code>startAngle</code>.
	 */
	public static native void drawThickCircleArc(byte[] gc, int x, int y, int diameter, float startAngle,
			float arcAngle, int thickness);

	/**
	 * LLGRAPHICS_EXT_IMPL_drawThickCircle
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param thickness
	 *            {@link ShapePainter#getThickness()}
	 * @param x
	 *            the x coordinate of the upper-left corner of the rectangle where the circle is drawn.
	 * @param y
	 *            the y coordinate of the upper-left corner of the rectangle where the circle is drawn.
	 * @param diameter
	 *            the diameter of the circle to draw.
	 */
	public static native void drawThickCircle(byte[] gc, int x, int y, int diameter, int thickness);

	/**
	 * LLGRAPHICS_EXT_IMPL_drawThickEllipse
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param thickness
	 *            {@link ShapePainter#getThickness()}
	 * @param x
	 *            the x coordinate of the upper-left corner of the rectangle where the circle is drawn.
	 * @param y
	 *            the y coordinate of the upper-left corner of the rectangle where the circle is drawn.
	 * @param diameter
	 *            the diameter of the circle to draw.
	 */
	public static native void drawThickEllipse(byte[] gc, int x, int y, int width, int height, int thickness);

	/**
	 * LLGRAPHICS_EXT_IMPL_drawDeformedImage
	 *
	 * @param destGC
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param srcImage
	 *            source context ({@link Image#getSNIContext()})
	 * @param xys
	 *            4 corners new position of left-top, right-top, right-bottom and left-bottom corners.
	 * @param workingBuffer
	 *            software algoithm working buffer
	 */
	public static native void drawDeformedImage(byte[] destGC, byte[] srcImage, int[] xys, byte[] workingBuffer);

	/**
	 * LLGRAPHICS_EXT_IMPL_scaleImageNearestNeighbor
	 *
	 * @param destGC
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param srcImage
	 *            source context ({@link Image#getSNIContext()})
	 * @param x
	 *            the x coordinate of upper-left corner
	 * @param y
	 *            the y coordinate of upper-left corner
	 * @param factorX
	 *            {@link Scale#getFactorX()}
	 * @param factorY
	 *            {@link Scale#getFactorY()}
	 * @param alpha
	 *            {@link Scale#getAlpha()}
	 * @param mirror
	 *            {@link Scale#isMirrored()}
	 */
	public static native void drawScaledImageNearestNeighbor(byte[] destGC, byte[] srcImage, int x, int y,
			float factorX,
			float factorY, int alpha);

	/**
	 * LLGRAPHICS_EXT_IMPL_scaleImageBilinear
	 *
	 * @param destGC
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param srcImage
	 *            source context ({@link Image#getSNIContext()})
	 * @param x
	 *            the x coordinate of destination upper-left corner
	 * @param y
	 *            the y coordinate of destination upper-left corner
	 * @param factorX
	 *            {@link Scale#getFactorX()}
	 * @param factorY
	 *            {@link Scale#getFactorY()}
	 * @param alpha
	 *            {@link Scale#getAlpha()}
	 * @param mirror
	 *            {@link Scale#isMirrored()}
	 */
	public static native void drawScaledImageBilinear(byte[] destGC, byte[] srcImage, int x, int y, float factorX,
			float factorY, int alpha);

	/**
	 * LLGRAPHICS_EXT_IMPL_flipImage
	 *
	 * @param destGC
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param srcImage
	 *            source context ({@link Image#getSNIContext()})
	 * @param xSrc
	 *            the x coordinate of source upper-left corner
	 * @param ySrc
	 *            the y coordinate of source upper-left corner
	 * @param width
	 *            the region width to flip
	 * @param height
	 *            the region height to flip
	 * @param xDest
	 *            the x coordinate of destination upper-left corner
	 * @param yDest
	 *            the y coordinate of destination upper-left corner
	 * @param transformation
	 *            {@link Flip#getAction()}
	 * @param alpha
	 *            {@link Flip#getAlpha()}
	 */
	public static native void drawFlippedImage(byte[] destGC, byte[] srcImage, int xSrc, int ySrc, int width, int height,
			int xDest, int yDest, int transformation, int alpha);

	/**
	 * LLGRAPHICS_EXT_IMPL_rotateImageBilinear
	 *
	 * @param destGC
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param srcImage
	 *            source context ({@link Image#getSNIContext()})
	 * @param destX
	 *            the x coordinate of destination upper-left corner
	 * @param destY
	 *            the y coordinate of destination upper-left corner
	 * @param xRotation
	 *            {@link Rotate#getRotationX()}
	 * @param yRotation
	 *            {@link Rotate#getRotationX()}
	 * @param angle
	 *            {@link Rotate#getAngle()}
	 * @param alpha
	 *            {@link Rotate#getAlpha()}}
	 * @param mirror
	 *            {@link Rotate#isMirrored()}
	 */
	public static native void drawRotatedImageBilinear(byte[] destGC, byte[] srcImage, int destX, int destY,
			int xRotation,
			int yRotation, float angle, int alpha);

	/**
	 * LLGRAPHICS_EXT_IMPL_rotateImageNearestNeighbor
	 *
	 * @param destGC
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param srcImage
	 *            source context ({@link Image#getSNIContext()})
	 * @param destX
	 *            the x coordinate of destination upper-left corner
	 * @param destY
	 *            the y coordinate of destination upper-left corner
	 * @param xRotation
	 *            {@link Rotate#getRotationX()}
	 * @param yRotation
	 *            {@link Rotate#getRotationX()}
	 * @param angle
	 *            {@link Rotate#getAngle()}
	 * @param alpha
	 *            {@link Rotate#getAlpha()}}
	 * @param mirror
	 *            {@link Rotate#isMirrored()}
	 */
	public static native void drawRotatedImageNearestNeighbor(byte[] destGC, byte[] srcImage, int destX, int destY,
			int xRotation, int yRotation, float angle, int alpha);

	/**
	 * LLGRAPHICS_EXT_IMPL_rotateCharacterNearestNeighbor
	 *
	 * @param sd
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param fontData
	 *            font native data
	 * @param c
	 *            character to draw
	 * @param destX
	 *            the x coordinate of destination upper-left corner
	 * @param destY
	 *            the y coordinate of destination upper-left corner
	 * @param xRotation
	 *            {@link Rotate#getRotationX()}
	 * @param yRotation
	 *            {@link Rotate#getRotationX()}
	 * @param angle
	 *            {@link Rotate#getAngle()}
	 * @param alpha
	 *            {@link Rotate#getAlpha()}}
	 * @param mirror
	 *            {@link Rotate#isMirrored()}
	 * @param externalFontHeap
	 *            working buffer when font is an external font
	 * @return error code | expected external font's working buffer size
	 */
	public static native int drawCharWithRotationNN(byte[] sd, byte[] fontData, char c, int destX, int destY,
			int xRotation, int yRotation, float angle, int alpha, byte[] externalFontHeap);

	/**
	 * LLGRAPHICS_EXT_IMPL_rotateCharacterBilinear
	 *
	 * @param sd
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param fontData
	 *            font native data
	 * @param c
	 *            character to draw
	 * @param destX
	 *            the x coordinate of destination upper-left corner
	 * @param destY
	 *            the y coordinate of destination upper-left corner
	 * @param xRotation
	 *            {@link Rotate#getRotationX()}
	 * @param yRotation
	 *            {@link Rotate#getRotationX()}
	 * @param angle
	 *            {@link Rotate#getAngle()}
	 * @param alpha
	 *            {@link Rotate#getAlpha()}}
	 * @param mirror
	 *            {@link Rotate#isMirrored()}
	 * @param externalFontHeap
	 *            working buffer when font is an external font
	 * @return error code | expected external font's working buffer size
	 */
	public static native int drawCharWithRotationBil(byte[] sd, byte[] fontData, char c, int destX, int destY,
			int xRotation, int yRotation, float angle, int alpha, byte[] externalFontHeap);

	/**
	 * LLGRAPHICS_EXT_IMPL_drawPolygon
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param xys
	 *            the array of coordinates : x1,y1,......xn,yn.
	 * @param length
	 *            the array's length of coordinates
	 */
	public static native void drawPolygon(byte[] gc, int[] xys, int length);

	/**
	 * LLGRAPHICS_EXT_IMPL_fillPolygon
	 *
	 * @param gc
	 *            destination context ({@link GraphicsContext#getSNIContext()})
	 * @param xys
	 *            the array of coordinates : x1,y1,......xn,yn.
	 * @param length
	 *            the array's length of coordinates
	 * @param workingBuffer
	 *            software algoithm working buffer
	 */
	public static native void fillPolygon(byte[] gc, int[] xys, int length, byte[] workingBuffer);

	/**
	 * Polygon (and deformImage) native algo requires some bytes to allocate its buffers:<br>
	 * = sizeOf(EdgeSP) * nbEdges * 2 + nbEdges*4 * 4 + sizeOf(PolygonSP)<br>
	 * = 40 * def * 2 + def * 4 * 4 + 20<br>
	 * = 96 * def + 20<br>
	 *
	 * @param nbEdges
	 *            number of edges to render
	 * @return the fill polygon (and deformImage) working buffer
	 */
	public static byte[] getPolygonWorkingBuffer(int nbEdges) {
		return new byte[96 * nbEdges + 20];
	}

	/**
	 * Check the error returned during a font use: drawText etc.
	 *
	 * @param ret
	 *            the potential error or the original return code
	 * @return the original return code when there is no error
	 * @throws MicroUIException
	 *             when an error is detected
	 */
	public static int checkFontRuntimeError(int ret) {

		/*
		 * WARNING this code is duplicated in microui (class Font)
		 */

		if (ret == INVALID_FILE) {
			// error when reading the font
			throw new MicroUIException(MicroUIException.RESOURCE_INVALID_FILE);
		} else if (ret == FILE_CLOSED) {
			// font (master font data) has been closed
			throw new MicroUIException(MicroUIException.RESOURCE_CLOSED);
		}
		return ret;
	}
}
