/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.drawing;

import ej.annotation.Nullable;
import ej.drawing.DrawingPainterNatives;
import ej.microui.display.GraphicsContext;
import ej.microui.display.Painter;

/**
 * microUI-API
 */
public class ShapePainter {

	/**
	 * microUI-API
	 */
	public enum Cap {
		/**
		 * microUI-API
		 */
		NONE(0),

		/**
		 * microUI-API
		 */
		ROUNDED(1),

		/**
		 * microUI-API
		 */
		PERPENDICULAR(2),

		/**
		 * microUI-API
		 */
		HORIZONTAL(3),

		/**
		 * microUI-API
		 */
		VERTICAL(4);

		private final int val;

		private Cap(int val) {
			this.val = val;
		}
	}

	private ShapePainter() {
		// static methods only
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param thickness
	 *            microUI-API
	 * @param fade
	 *            microUI-API
	 */
	public static void drawThickFadedPoint(GraphicsContext gc, int x, int y, int thickness, int fade) {
		if (thickness < 0 || fade < 0) {
			throw new IllegalArgumentException();
		}
		int translateX = gc.getTranslateX();
		int translateY = gc.getTranslateY();
		DrawingPainterNatives.drawThickFadedPoint(gc.getSNIContext(), x + translateX, y + translateY, thickness, fade);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x1
	 *            microUI-API
	 * @param y1
	 *            microUI-API
	 * @param x2
	 *            microUI-API
	 * @param y2
	 *            microUI-API
	 * @param thickness
	 *            microUI-API
	 */
	public static void drawThickLine(GraphicsContext gc, int x1, int y1, int x2, int y2, int thickness) {
		if (thickness < 0) {
			throw new IllegalArgumentException();
		}
		int translateX = gc.getTranslateX();
		int translateY = gc.getTranslateY();
		DrawingPainterNatives.drawThickLine(gc.getSNIContext(), x1 + translateX, y1 + translateY, x2 + translateX,
				y2 + translateY, thickness);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x1
	 *            microUI-API
	 * @param y1
	 *            microUI-API
	 * @param x2
	 *            microUI-API
	 * @param y2
	 *            microUI-API
	 * @param thickness
	 *            microUI-API
	 * @param fade
	 *            microUI-API
	 * @param startCap
	 *            microUI-API
	 * @param endCap
	 *            microUI-API
	 */
	public static void drawThickFadedLine(GraphicsContext gc, int x1, int y1, int x2, int y2, int thickness, int fade,
			Cap startCap,
			Cap endCap) {
		if (thickness < 0 || fade < 0) {
			throw new IllegalArgumentException();
		}
		int translateX = gc.getTranslateX();
		int translateY = gc.getTranslateY();
		DrawingPainterNatives.drawThickFadedLine(gc.getSNIContext(), x1 + translateX, y1 + translateY, x2 + translateX,
				y2 + translateY, thickness, fade, startCap.val, endCap.val /* not in api yet */);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param diameter
	 *            microUI-API
	 * @param startAngle
	 *            microUI-API
	 * @param arcAngle
	 *            microUI-API
	 * @param thickness
	 *            microUI-API
	 */
	public static void drawThickCircleArc(GraphicsContext gc, int x, int y, int diameter, float startAngle,
			float arcAngle,
			int thickness) {
		if (thickness < 0) {
			throw new IllegalArgumentException();
		}
		int translateX = gc.getTranslateX();
		int translateY = gc.getTranslateY();
		DrawingPainterNatives.drawThickCircleArc(gc.getSNIContext(), translateX + x, translateY + y, diameter,
				startAngle, arcAngle, thickness);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param diameter
	 *            microUI-API
	 * @param startAngle
	 *            microUI-API
	 * @param arcAngle
	 *            microUI-API
	 * @param thickness
	 *            microUI-API
	 * @param fade
	 *            microUI-API
	 * @param startCap
	 *            microUI-API
	 * @param endCap
	 *            microUI-API
	 */
	public static void drawThickFadedCircleArc(GraphicsContext gc, int x, int y, int diameter, float startAngle,
			float arcAngle, int thickness, int fade, Cap startCap, Cap endCap) {
		if (thickness < 0 || fade < 0) {
			throw new IllegalArgumentException();
		}
		int translateX = gc.getTranslateX();
		int translateY = gc.getTranslateY();
		DrawingPainterNatives.drawThickFadedCircleArc(gc.getSNIContext(), translateX + x, translateY + y, diameter,
				startAngle, arcAngle, thickness, fade, startCap.val, endCap.val /* not in api yet */);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-APImicroUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param diameter
	 *            microUI-API
	 * @param thickness
	 *            microUI-API
	 */
	public static void drawThickCircle(GraphicsContext gc, int x, int y, int diameter, int thickness) {
		if (thickness < 0) {
			throw new IllegalArgumentException();
		}
		int translateX = gc.getTranslateX();
		int translateY = gc.getTranslateY();
		DrawingPainterNatives.drawThickCircle(gc.getSNIContext(), translateX + x, translateY + y, diameter, thickness);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param diameter
	 *            microUI-API
	 * @param thickness
	 *            microUI-API
	 * @param fade
	 *            microUI-API
	 */
	public static void drawThickFadedCircle(GraphicsContext gc, int x, int y, int diameter, int thickness, int fade) {
		if (thickness < 0 || fade < 0) {
			throw new IllegalArgumentException();
		}
		int translateX = gc.getTranslateX();
		int translateY = gc.getTranslateY();
		DrawingPainterNatives.drawThickFadedCircle(gc.getSNIContext(), translateX + x, translateY + y, diameter,
				thickness, fade);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 * @param thickness
	 *            microUI-API
	 */
	public static void drawThickEllipse(GraphicsContext gc, int x, int y, int width, int height, int thickness) {
		if (thickness < 0) {
			throw new IllegalArgumentException();
		}
		int translateX = gc.getTranslateX();
		int translateY = gc.getTranslateY();
		DrawingPainterNatives.drawThickEllipse(gc.getSNIContext(), translateX + x, translateY + y, width, height,
				thickness);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 * @param thickness
	 *            microUI-API
	 * @param fade
	 *            microUI-API
	 */
	public static void drawThickFadedEllipse(GraphicsContext gc, int x, int y, int width, int height, int thickness,
			int fade) {
		if (thickness < 0 || fade < 0) {
			throw new IllegalArgumentException();
		}
		int translateX = gc.getTranslateX();
		int translateY = gc.getTranslateY();
		DrawingPainterNatives.drawThickFadedEllipse(gc.getSNIContext(), translateX + x, translateY + y, width, height,
				thickness, fade);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param xys
	 *            microUI-API
	 */
	public static void drawPolygon(GraphicsContext gc, int[] xys) {
		drawPolygon(gc, xys, 0, xys.length);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param xys
	 *            microUI-API
	 * @param offset
	 *            microUI-API
	 * @param length
	 *            microUI-API
	 */
	public static void drawPolygon(GraphicsContext gc, int[] xys, int offset, int length) {
		xys = check(gc, xys, offset, length);
		if (xys != null) {
			DrawingPainterNatives.drawPolygon(gc.getSNIContext(), xys, xys.length);
		}
		// else: nothing to draw
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param xys
	 *            microUI-API
	 */
	public static void fillPolygon(GraphicsContext gc, int[] xys) {
		fillPolygon(gc, xys, 0, xys.length);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param xys
	 *            microUI-API
	 * @param offset
	 *            microUI-API
	 * @param length
	 *            microUI-API
	 */
	public static void fillPolygon(GraphicsContext gc, int[] xys, int offset, int length) {
		xys = check(gc, xys, offset, length);
		if (xys != null) {
			byte[] workingBuffer = DrawingPainterNatives.getPolygonWorkingBuffer(length / 2);
			DrawingPainterNatives.fillPolygon(gc.getSNIContext(), xys, xys.length, workingBuffer);
		}
		// else: nothing to draw
	}

	/**
	 * Check the xys array validity. if array is invalid, throw the right exception. if the array is smaller than 6,
	 * draw the pixel, the line or nothing and returns false. otherwise returns true;<br>
	 */
	private static @Nullable int[] check(GraphicsContext gc, int[] xys, int offset, int length) {

		checkArrayBounds(xys.length, offset, length);

		if ((length & 1) != 0) {
			throw new IllegalArgumentException();
		}

		if (length == 0) {
			// nothing to draw !
			return null;
		} else if (length == 2) {
			// draw a pixel
			Painter.drawPixel(gc, xys[0], xys[1]);
			return null;
		} else if (length == 4) {
			// draw a line
			Painter.drawLine(gc, xys[0], xys[1], xys[2], xys[3]);
			return null;
		}

		// copy to the working array (do not modify caller array)
		int[] out = new int[length];
		System.arraycopy(xys, offset, out, 0, length);

		// apply translation
		int tx = gc.getTranslateX();
		int ty = gc.getTranslateY();
		if (tx != 0 || ty != 0) {
			for (int i = 0; i < length; i += 2) {
				out[i] += tx;
				out[i + 1] += ty;
			}
		}

		// drawing has to be done
		return out;
	}

	private static void checkArrayBounds(int arrayLength, int off, int len) {
		int offPlusLen = off + len;
		if (off < 0 || len < 0 || offPlusLen > arrayLength || offPlusLen < 0) {
			throw new ArrayIndexOutOfBoundsException();
		}
	}
}
