
/*
 *	Java
 *
 *	Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 *	This library is provided in source code for use, modification and test, subject to license terms.
 *	Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */

package ej.drawing;

import ej.drawing.DrawingPainterNatives;
import ej.microui.display.GraphicsContext;
import ej.microui.display.Image;
import ej.microui.display.RenderableChar;

/**
 * microUI-API
 */
public class TransformPainter {

	/**
	 * microUI-API
	 */
	public enum Flip {

		/**
		 * microUI-API
		 */
		FLIP_NONE(0),

		/**
		 * microUI-API
		 */
		FLIP_90(7),

		/**
		 * microUI-API
		 */
		FLIP_180(4),

		/**
		 * microUI-API
		 */
		FLIP_270(2),

		/**
		 * microUI-API
		 */
		FLIP_MIRROR(3),

		/**
		 * microUI-API
		 */
		FLIP_MIRROR_90(1),

		/**
		 * microUI-API
		 */
		FLIP_MIRROR_180(6),

		/**
		 * microUI-API
		 */
		FLIP_MIRROR_270(5);

		/* default */ int val;

		private Flip(int val) {
			this.val = val;
		}
	}

	private TransformPainter() {
		// static methods only
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param flip
	 *            microUI-API
	 */
	public static void drawFlippedImage(GraphicsContext gc, Image image, int x, int y, Flip flip) {
		drawFlippedImage(gc, image, x, y, flip, GraphicsContext.OPAQUE);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param flip
	 *            microUI-API
	 * @param alpha
	 *            microUI-API
	 */
	public static void drawFlippedImage(GraphicsContext gc, Image image, int x, int y, Flip flip, int alpha) {
		DrawingPainterNatives.drawFlippedImage(gc.getSNIContext(), image.getSNIContext(), 0, 0, image.getWidth(), image.getHeight(),
				getX(gc, x), getY(gc, y), flip.val, alpha);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param xys
	 *            microUI-API
	 */
	public static void drawDeformedImage(GraphicsContext gc, Image image, int x, int y, int[] xys) {

		int count = xys.length;
		if (count != 4 * 2) {
			throw new IllegalArgumentException();
		}

		int[] xy = new int[4 * 2];
		synchronized (xy) {

			for (int i = count; --i >= 0;) {
				xy[i] = getY(gc, xys[i]) + y;
				xy[--i] = getX(gc, xys[i]) + x;
			}

			byte[] workingBuffer = DrawingPainterNatives.getPolygonWorkingBuffer(4);
			DrawingPainterNatives.drawDeformedImage(image.getSNIContext(), gc.getSNIContext(), xy, workingBuffer);
		}
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param rx
	 *            microUI-API
	 * @param ry
	 *            microUI-API
	 * @param angle
	 *            microUI-API
	 */
	public static void drawRotatedImageBilinear(GraphicsContext gc, Image image, int x, int y, int rx, int ry,
			float angle) {
		DrawingPainterNatives.drawRotatedImageBilinear(gc.getSNIContext(), image.getSNIContext(), getX(gc, x),
				getY(gc, y),
				getX(gc, rx), getY(gc, ry), angle,
				GraphicsContext.OPAQUE);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param rx
	 *            microUI-API
	 * @param ry
	 *            microUI-API
	 * @param angle
	 *            microUI-API
	 * @param alpha
	 *            microUI-API
	 */
	public static void drawRotatedImageBilinear(GraphicsContext gc, Image image, int x, int y, int rx, int ry,
			float angle,
			int alpha) {
		DrawingPainterNatives.drawRotatedImageBilinear(gc.getSNIContext(), image.getSNIContext(), getX(gc, x),
				getY(gc, y),
				getX(gc, rx), getY(gc, ry), angle, alpha);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param rx
	 *            microUI-API
	 * @param ry
	 *            microUI-API
	 * @param angle
	 *            microUI-API
	 */
	public static void drawRotatedImageNearestNeighbor(GraphicsContext gc, Image image, int x, int y, int rx, int ry,
			float angle) {
		DrawingPainterNatives.drawRotatedImageNearestNeighbor(gc.getSNIContext(), image.getSNIContext(), getX(gc, x),
				getY(gc, y),
				getX(gc, rx), getY(gc, ry), angle,
				GraphicsContext.OPAQUE);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param rx
	 *            microUI-API
	 * @param ry
	 *            microUI-API
	 * @param angle
	 *            microUI-API
	 * @param alpha
	 *            microUI-API
	 */
	public static void drawRotatedImageNearestNeighbor(GraphicsContext gc, Image image, int x, int y, int rx, int ry,
			float angle,
			int alpha) {
		DrawingPainterNatives.drawRotatedImageNearestNeighbor(gc.getSNIContext(), image.getSNIContext(), getX(gc, x),
				getY(gc, y),
				getX(gc, rx), getY(gc, ry), angle, alpha);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param character
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param rx
	 *            microUI-API
	 * @param ry
	 *            microUI-API
	 * @param angle
	 *            microUI-API
	 */
	public static void drawRotatedRenderableCharBilinear(GraphicsContext gc, RenderableChar character, int x, int y,
			int rx, int ry, float angle) {
		drawRotatedRenderableCharBilinear(gc, character, x, y, rx, ry, angle, GraphicsContext.OPAQUE);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param character
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param rx
	 *            microUI-API
	 * @param ry
	 *            microUI-API
	 * @param angle
	 *            microUI-API
	 * @param alpha
	 *            microUI-API
	 */
	public static void drawRotatedRenderableCharBilinear(GraphicsContext gc, RenderableChar character, int x, int y,
			int rx, int ry, float angle, int alpha) {
		byte[] fontData = character.getFont().getSNIContext();
		char c = character.getChar();
		byte[] dest = gc.getSNIContext();
		int dx = getX(gc, x);
		int dy = getY(gc, y);
		int xRotation = getX(gc, rx);
		int yRotation = getY(gc, ry);

		int workingBufferSize = DrawingPainterNatives.checkFontRuntimeError(DrawingPainterNatives.drawCharWithRotationBil(
				dest, fontData, c, dx, dy, xRotation, yRotation, angle, alpha, null));

		if (workingBufferSize > 0) {
			// requires a working buffer to perform the operation
			DrawingPainterNatives.checkFontRuntimeError(DrawingPainterNatives.drawCharWithRotationBil(dest, fontData, c,
					dx, dy, xRotation, yRotation, angle, alpha, new byte[workingBufferSize]));
		}
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param character
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param rx
	 *            microUI-API
	 * @param ry
	 *            microUI-API
	 * @param angle
	 *            microUI-API
	 */
	public static void drawRotatedRenderableCharNearestNeighbor(GraphicsContext gc, RenderableChar character, int x,
			int y, int rx, int ry, float angle) {
		drawRotatedRenderableCharNearestNeighbor(gc, character, x, y, rx, ry, angle, GraphicsContext.OPAQUE);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param character
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param rx
	 *            microUI-API
	 * @param ry
	 *            microUI-API
	 * @param angle
	 *            microUI-API
	 * @param alpha
	 *            microUI-API
	 */
	public static void drawRotatedRenderableCharNearestNeighbor(GraphicsContext gc, RenderableChar character, int x,
			int y, int rx, int ry, float angle, int alpha) {
		byte[] fontData = character.getFont().getSNIContext();
		char c = character.getChar();
		byte[] dest = gc.getSNIContext();
		int dx = getX(gc, x);
		int dy = getY(gc, y);
		int xRotation = getX(gc, rx);
		int yRotation = getY(gc, ry);

		int workingBufferSize = DrawingPainterNatives.checkFontRuntimeError(DrawingPainterNatives.drawCharWithRotationNN(
				dest, fontData, c, dx, dy, xRotation, yRotation, angle, alpha, null));

		if (workingBufferSize > 0) {
			// requires a working buffer to perform the operation
			DrawingPainterNatives.checkFontRuntimeError(DrawingPainterNatives.drawCharWithRotationNN(dest, fontData, c,
					dx, dy, xRotation, yRotation, angle, alpha, new byte[workingBufferSize]));
		}
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param factorX
	 *            microUI-API
	 * @param factorY
	 *            microUI-API
	 */
	public static void drawScaledImageBilinear(GraphicsContext gc, Image image, int x, int y, float factorX,
			float factorY) {
		checkScaleFactors(factorX, factorY);
		DrawingPainterNatives.drawScaledImageBilinear(gc.getSNIContext(), image.getSNIContext(), getX(gc, x),
				getY(gc, y), factorX, factorY, GraphicsContext.OPAQUE);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param factorX
	 *            microUI-API
	 * @param factorY
	 *            microUI-API
	 * @param alpha
	 *            microUI-API
	 */
	public static void drawScaledImageBilinear(GraphicsContext gc, Image image, int x, int y, float factorX,
			float factorY, int alpha) {
		checkScaleFactors(factorX, factorY);
		DrawingPainterNatives.drawScaledImageBilinear(gc.getSNIContext(), image.getSNIContext(), getX(gc, x),
				getY(gc, y), factorX, factorY, alpha);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param factorX
	 *            microUI-API
	 * @param factorY
	 *            microUI-API
	 */
	public static void drawScaledImageNearestNeighbor(GraphicsContext gc, Image image, int x, int y, float factorX,
			float factorY) {
		checkScaleFactors(factorX, factorY);
		DrawingPainterNatives.drawScaledImageNearestNeighbor(gc.getSNIContext(), image.getSNIContext(), getX(gc, x),
				getY(gc, y), factorX, factorY, GraphicsContext.OPAQUE);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param factorX
	 *            microUI-API
	 * @param factorY
	 *            microUI-API
	 * @param alpha
	 *            microUI-API
	 */
	public static void drawScaledImageNearestNeighbor(GraphicsContext gc, Image image, int x, int y, float factorX,
			float factorY, int alpha) {
		checkScaleFactors(factorX, factorY);
		DrawingPainterNatives.drawScaledImageNearestNeighbor(gc.getSNIContext(), image.getSNIContext(), getX(gc, x),
				getY(gc, y), factorX, factorY, alpha);
	}

	private static void checkScaleFactors(float factorX, float factorY) {
		if (factorX <= 0.0f || factorY <= 0.0f) {
			throw new IllegalArgumentException();
		}
	}

	/**
	 * @return the given coordinate shifted with user translation AND {@link GraphicsContext} translation
	 */
	private static int getX(GraphicsContext gc, int x) {
		return x + gc.getTranslateX();
	}

	/**
	 * @return the given coordinate shifted with user translation AND {@link GraphicsContext} translation
	 */
	private static int getY(GraphicsContext gc, int y) {
		return y + gc.getTranslateY();
	}
}
