/*
 * Java
 *
 * Copyright 2016-2019 MicroEJ Corp. All rights reserved.
 * MicroEJ Corp. PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */
package ej.giml.annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * An annotation to tag constructor parameters, in order to declare them as GIML attributes of their parent GIML tag.
 * Element attributes can have default value.
 *
 * @see ElementRequiredAttribute
 */
@Target(ElementType.PARAMETER)
@Retention(RetentionPolicy.SOURCE)
public @interface ElementAttribute {
	/**
	 * The default value of this attribute. For {@link String} attributes, values must be preceded by / end with
	 * {@code \"}.
	 *
	 * @return the default value of this attribute.
	 */
	String defaultValue();

	/**
	 * The GIML attribute name. Empty name means to use the parameter name as attribute name.
	 *
	 * @return the name of this attribute.
	 */
	String name() default "";

	/**
	 * Defines if this attribute is a reference. If {@code true}, the type of this attribute is set to {@code reference}
	 * instead of its true type.
	 *
	 * @return {@code true} if this attribute is a reference, {@code false} otherwise.
	 */
	boolean reference() default false;
}
