
/*
 *	Java
 *
 *	Copyright 2015-2020 MicroEJ Corp. All rights reserved.
 *	This library is provided in source code for use, modification and test, subject to license terms.
 *	Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */

package ej.microui.display;

import com.is2t.microbsp.microui.natives.NSystemDisplay;

import ej.microui.Log;
import ej.microui.MicroUI;
import ej.microui.MicroUIException;

/**
 * microUI-API
 */
public class BufferedImage extends ResourceImage {

	private final MutableImageGraphicsContext gc;

	/**
	 * microUI-API
	 *
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 */
	public BufferedImage(int width, int height) {
		super(openOffscreen(width, height), Log.TYPE_OPEN_IMAGE_MUTABLE, Log.IMAGE_CARACTERISTICS_NEW_IMAGE);
		this.gc = new MutableImageGraphicsContext(this);
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public GraphicsContext getGraphicsContext() {
		if (isClosed()) {
			throw new MicroUIException(MicroUIException.RESOURCE_CLOSED);
		}
		// if a image.close() is called here, gc is returned but the "close" error will
		// occur later
		return this.gc;
	}

	/**
	 * Malloc a space to store an offscreen in native world and return his id.
	 *
	 * @param destSd
	 *            the pointer on the screen descriptor where new offscreen will be
	 *            drawn
	 * @param width
	 *            the offscreen width
	 * @param height
	 *            the offscreen height
	 * @return the offscreen id
	 * @throws MicroUIException
	 *             if there is not enough room to add a new offscreen.
	 */
	private static byte[] openOffscreen(int width, int height) {
		MicroUI.checkRunning();
		logCreateImageStart(Log.TYPE_OPEN_IMAGE_MUTABLE);
		checkSecurityManager();

		if (width <= 0 || height <= 0) {
			throw new IllegalArgumentException();
		}

		// try to open the offscreen. A positive returned value is the offscreen id.
		byte[] sd = SImageMetadata.allocate(true);
		NSystemDisplay.checkError(NSystemDisplay.openOffScreen(width, height, sd), null);
		return sd;
	}
}
