/*
 * Java
 *
 * Copyright 2008-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.microui.display;

import java.io.IOException;

import com.is2t.microbsp.microui.natives.NSystemDisplay;
import com.is2t.tools.ArrayTools;

import ej.bon.Constants;
import ej.bon.Immortals;
import ej.lang.Resource;
import ej.lang.ResourceManager;
import ej.microui.MicroUI;
import ej.microui.MicroUIException;
import ej.microui.MicroUIProperties;

/**
 * microUI-API
 */
public class Font {

	/**
	 * microUI-API
	 */
	public static final int STYLE_PLAIN = 0;

	/**
	 * microUI-API
	 */
	public static final int STYLE_BOLD = 1;

	/**
	 * microUI-API
	 */
	public static final int STYLE_ITALIC = 1 << 1;

	/**
	 * microUI-API
	 */
	public static final int ARABIC = 1;

	/**
	 * microUI-API
	 */
	public static final int ARMENIAN = 2;

	/**
	 * microUI-API
	 */
	public static final int BALINESE = 3;

	/**
	 * microUI-API
	 */
	public static final int BENGALI = 4;

	/**
	 * microUI-API
	 */
	public static final int BOPOMOFO = 5;

	/**
	 * microUI-API
	 */
	public static final int BRAILLE = 6;

	/**
	 * microUI-API
	 */
	public static final int BUGINESE = 7;

	/**
	 * microUI-API
	 */
	public static final int BUHID = 8;

	/**
	 * microUI-API
	 */
	public static final int CANADIAN_ABORIGINAL = 9;

	/**
	 * microUI-API
	 */
	public static final int CHEROKEE = 10;

	/**
	 * microUI-API
	 */
	public static final int COPTIC = 11;

	/**
	 * microUI-API
	 */
	public static final int CYRILLIC = 14;

	/**
	 * microUI-API
	 */
	public static final int DEVANAGARI = 16;

	/**
	 * microUI-API
	 */
	public static final int ETHIOPIC = 17;

	/**
	 * microUI-API
	 */
	public static final int GEORGIAN = 18;

	/**
	 * microUI-API
	 */
	public static final int GLAGOLITIC = 19;

	/**
	 * microUI-API
	 */
	public static final int GREEK = 21;

	/**
	 * microUI-API
	 */
	public static final int GUJARATI = 22;

	/**
	 * microUI-API
	 */
	public static final int GURMUKHI = 23;

	/**
	 * microUI-API
	 */
	public static final int HAN = 24;

	/**
	 * microUI-API
	 */
	public static final int HANGUL = 25;

	/**
	 * microUI-API
	 */
	public static final int HANUNOO = 26;

	/**
	 * microUI-API
	 */
	public static final int HEBREW = 27;

	/**
	 * microUI-API
	 */
	public static final int HIRAGANA = 28;

	/**
	 * microUI-API
	 */
	public static final int KANNADA = 29;

	/**
	 * microUI-API
	 */
	public static final int KATAKANA = 30;

	/**
	 * microUI-API
	 */
	public static final int KHMER = 32;

	/**
	 * microUI-API
	 */
	public static final int LAO = 33;

	/**
	 * microUI-API
	 */
	public static final int LATIN = 34;

	/**
	 * microUI-API
	 */
	public static final int LIMBU = 35;

	/**
	 * microUI-API
	 */
	public static final int MALAYALAM = 36;

	/**
	 * microUI-API
	 */
	public static final int MONGOLIAN = 37;

	/**
	 * microUI-API
	 */
	public static final int MYANMAR = 38;

	/**
	 * microUI-API
	 */
	public static final int NEW_TAI_LUE = 39;

	/**
	 * microUI-API
	 */
	public static final int NKO = 40;

	/**
	 * microUI-API
	 */
	public static final int OGHAM = 41;

	/**
	 * microUI-API
	 */
	public static final int ORIYA = 42;

	/**
	 * microUI-API
	 */
	public static final int PHAGS_PA = 44;

	/**
	 * microUI-API
	 */
	public static final int RUNIC = 46;

	/**
	 * microUI-API
	 */
	public static final int SINHALA = 48;

	/**
	 * microUI-API
	 */
	public static final int SYLOTI_NAGRI = 49;

	/**
	 * microUI-API
	 */
	public static final int SYRIAC = 50;

	/**
	 * microUI-API
	 */
	public static final int TAGALOG = 51;

	/**
	 * microUI-API
	 */
	public static final int TAGBANWA = 52;

	/**
	 * microUI-API
	 */
	public static final int TAI_LE = 53;

	/**
	 * microUI-API
	 */
	public static final int TAMIL = 54;

	/**
	 * microUI-API
	 */
	public static final int TELUGU = 55;

	/**
	 * microUI-API
	 */
	public static final int THAANA = 56;

	/**
	 * microUI-API
	 */
	public static final int THAI = 57;

	/**
	 * microUI-API
	 */
	public static final int TIBETAN = 58;

	/**
	 * microUI-API
	 */
	public static final int TIFINAGH = 59;

	/**
	 * microUI-API
	 */
	public static final int YI = 61;

	/**
	 * microUI-API
	 */
	public static final int COMMON = 62;

	/**
	 * microUI-API
	 */
	public static final int INHERITED = 63;

	/**
	 * microUI-API
	 */
	public static final int SUNDANESE = 64;

	/**
	 * microUI-API
	 */
	public static final int LEPCHA = 65;

	/**
	 * microUI-API
	 */
	public static final int OL_CHIKI = 66;

	/**
	 * microUI-API
	 */
	public static final int VAI = 67;

	/**
	 * microUI-API
	 */
	public static final int SAURASHTRA = 68;

	/**
	 * microUI-API
	 */
	public static final int KAYAH_LI = 69;

	/**
	 * microUI-API
	 */
	public static final int REJANG = 70;

	/**
	 * microUI-API
	 */
	public static final int CHAM = 71;

	/**
	 * microUI-API
	 */
	public static final int TAI_THAM = 72;

	/**
	 * microUI-API
	 */
	public static final int TAI_VIET = 73;

	/**
	 * microUI-API
	 */
	public static final int SAMARITAN = 74;

	/**
	 * microUI-API
	 */
	public static final int LISU = 75;

	/**
	 * microUI-API
	 */
	public static final int BAMUM = 76;

	/**
	 * microUI-API
	 */
	public static final int JAVANESE = 77;

	/**
	 * microUI-API
	 */
	public static final int MEETEI_MAYEK = 78;

	/**
	 * microUI-API
	 */
	public static final int BATAK = 79;

	/**
	 * microUI-API
	 */
	public static final int MANDAIC = 80;

	/**
	 * EJF masks and shifts
	 */
	private static final int FONT_STYLES_SHIFT = 4;
	private static final int FONT_STYLES_MASK = 0xf0;
	private static final int FONT_MASK_MONO = 0x100;
	private static final int FONT_MASK_BOLD = 0x1;
	private static final int FONT_MASK_ITALIC = 0x2;

	private static Font[] Fonts; // use getFonts()

	private final FontsData data;
	private final byte style;

	/**
	 * microUI-API
	 */
	private Font() {
		this((byte) 0, null);
	}

	private Font(byte style, FontsData data) {
		this.style = style;
		this.data = data;
	}

	/**
	 * microUI
	 *
	 * @return microUI-API
	 */
	public int[] getIdentifiers() {
		// spec does not specify if we have to return a copy
		return this.data.identifiers;
	}

	/**
	 * microUI-API
	 *
	 * @param identifier
	 *            microUI-API
	 * @return microUI-API
	 */
	public boolean isIdentifierSupported(int identifier) {
		int[] identifiers = getIdentifiers();
		for (int i = identifiers.length; --i >= 0;) {
			if (identifiers[i] == identifier) {
				return true;
			}
		}
		return false;
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public int getStyle() {
		return this.style & (STYLE_BOLD | STYLE_ITALIC | STYLE_PLAIN);
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public String getDescriptor() {
		return this.data.getFilename();

	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public boolean isPlain() {
		// the style is Plain if no BOLD and no Italic (see getStyle())
		return (this.style & (STYLE_BOLD | STYLE_ITALIC)) == 0;
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */

	public boolean isBold() {
		return (this.style & STYLE_BOLD) != 0;
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public boolean isItalic() {
		return (this.style & STYLE_ITALIC) != 0;
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public boolean isMonospaced() {
		return this.data.isMonospace();
	}

	/**
	 * microUI-API
	 *
	 * @param ch
	 *            microUI-API
	 * @return microUI-API
	 */
	public int charWidth(char ch) {
		return getWidth(new char[] { ch }, 0, 1, null);
	}

	/**
	 * microUI-API
	 *
	 * @param ch
	 *            microUI-API
	 * @param offset
	 *            microUI-API
	 * @param length
	 *            microUI-API
	 * @return microUI-API
	 */
	public int charsWidth(char[] ch, int offset, int length) {
		// check bounds, throw ArrayIndexOutOfBoundsException if wrong values
		// ch.length can throw NPE
		ArrayTools.checkArrayBounds(ch.length, offset, length);

		return getWidth(ch, offset, length, null);
	}

	/**
	 * microUI-API
	 *
	 * @param str
	 *            microUI-API
	 * @return microUI-API
	 */
	public int stringWidth(String str) {
		int length = str.length(); // can throw NPE (spec)
		return getWidth(str.chars, str.offset, length, null);
	}

	/**
	 * microUI-API
	 *
	 * @param str
	 *            microUI-API
	 * @param offset
	 *            microUI-API
	 * @param len
	 *            microUI-API
	 * @return microUI-API
	 */
	public int substringWidth(String str, int offset, int len) {
		try {
			// check bounds, throw ArrayIndexOutOfBoundsException if wrong
			// values; str.length() can throw NPE
			ArrayTools.checkArrayBounds(str.length(), offset, len);
		} catch (ArrayIndexOutOfBoundsException e) {
			throw new StringIndexOutOfBoundsException();
		}

		// perform this check AFTER checkArrayBounds (spec)
		return getWidth(str.chars, str.offset + offset, len, null);
	}

	/**
	 * @return native width (ratio is not applied)
	 */
	/* default */ int getWidth(char[] ch, int offset, int length, int[] indexes) {
		return Font.checkFontRuntimeError(NSystemDisplay.charsWidth(this.data.identifier, ch, offset, length, indexes,
				this.data.mallocExternalFontHeap()));
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public static Font getDefaultFont() {
		MicroUI.checkRunning();
		try {
			return getFontsInternal()[0];
		} catch (Exception e) {
			throw new MicroUIException(MicroUIException.NO_FONT);
		}
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public static Font[] getAllFonts() {
		// copy the array of fonts
		Font[] fonts = getFonts();
		int nbFonts = fonts.length;
		Font[] copy = new Font[nbFonts];
		System.arraycopy(fonts, 0, copy, 0, nbFonts);
		return copy;
	}

	/**
	 * microUI-API
	 *
	 * @param path
	 *            microUI-API
	 *
	 * @return microUI-API
	 */
	public static Font getFont(String path) {
		String p = MicroUIException.checkResourcePath(path);
		Font[] fonts = getFonts();
		for (Font font : fonts) {
			if (font.data.isSamePath(p)) {
				return font;
			}
		}
		throw new MicroUIException(MicroUIException.RESOURCE_INVALID_PATH, p);
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public int getHeight() {
		return this.data.height;
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public int getBaselinePosition() {
		return this.data.baseline;
	}

	/* default */ int getFontIdentifier() {
		return this.data.identifier;
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public byte[] getSNIContext() {
		return this.data.nativeData;
	}

	/* default */ byte[] mallocExternalFontHeap() {
		return this.data.mallocExternalFontHeap();
	}

	private static synchronized Font[] getFonts() {
		MicroUI.checkRunning();
		return getFontsInternal();
	}

	/**
	 * Lazy initialization of Fonts.
	 * <p>
	 * Synchronization must be ensured by the caller.
	 */
	/* default */ static Font[] getFontsInternal() {
		if (Fonts == null) {
			// No injection - default case
			Font.Fonts = initializeAllFonts();
		}
		return Fonts;
	}

	/* default */ static Font[] initializeAllFonts() {
		MicroUI.checkRunning();

		Font[] microUIFonts = addFonts(Constants.getString(MicroUIProperties.FONTS_LIST), new Font[0]);

		// first font is the default font
		microUIFonts = Immortals.setImmortal(microUIFonts);
		return microUIFonts;
	}

	private static void checkSecurityManager() {
		if (Constants.getBoolean(MicroUIProperties.CONSTANT_USE_SECURITYMANAGER)) {
			SecurityManager sm = System.getSecurityManager();
			if (sm != null) {
				sm.checkPermission(new FontPermission());
			}
		}
	}

	/* default */ static Font[] addFonts(final String list, Font[] fonts) {

		// the try-catch block should be written inside the if-constants block. But
		// the issue M0081MEJA-842 prevents it.
		// 96 bytes in flash can be won moving this try-catch.
		try {
			checkSecurityManager();
		} catch (SecurityException e) {
			// Not allowed to add font - silently complete
			return fonts;
		}

		// prepare the property
		String prop = list.trim();
		prop += MicroUIProperties.FONTS_LIST_SEPARATOR; // add it even if already exists

		// add all user fonts
		int index = 0;
		while (true) {

			// get the next index of ':'
			int sepIndex = prop.indexOf(MicroUIProperties.FONTS_LIST_SEPARATOR, index);

			// it was the last identifier -> go out
			if (sepIndex == -1) {
				break;
			}

			// get the font path
			String path = prop.substring(index, sepIndex);

			// add all MicroUI fonts for this path
			fonts = addFont(list, fonts, path);

			// update index
			index = sepIndex + 1;
		}

		return fonts;
	}

	private static Font[] addFont(final String list, Font[] microUIFonts, String path) {

		path = path.trim();
		if (path.length() == 0) {
			return microUIFonts; // empty
		}

		if (path.charAt(0) != '/') {
			MicroUI.MicroUI.errorLog(new MicroUIException(MicroUIException.RESOURCE_INVALID_PATH, path));
			return microUIFonts; // empty
		}

		try {

			// add NULL character at the end of string to be able load an external font
			String nativePath = path + '\0';

			// load font and retrieve some font characteristics
			byte[] nativeFontData = new byte[FontsData.LOAD_OFFSETS_ARRAY_SIZE];
			checkFontLoadingError(NSystemDisplay.addInternalFont(nativePath.getBytes(), nativePath.length, nativeFontData));

			// create a font data object for the ejf file. This object is shared
			// between each derived font
			FontsData fontData = new FontsData(path, nativeFontData, list);

			// add the shared resource to the resources manager
			ResourceManager rm = ResourceManager.getResourceManager();
			if (rm != null) {
				rm.resourceCreated(fontData);
			}

			// retrieve all derived fonts to create from styles
			int styles = getStyles(fontData.getStyles(), FONT_STYLES_MASK, FONT_STYLES_SHIFT);

			// get all available styles
			int style = 0;
			style |= getStyleFlag(styles, FONT_MASK_BOLD, STYLE_BOLD);
			style |= getStyleFlag(styles, FONT_MASK_ITALIC, STYLE_ITALIC);

			microUIFonts = (Font[]) ArrayTools.add(microUIFonts, new Font((byte) style, fontData));

		} catch (IOException e) {
			// just print a warning and try to load remaining fonts
			MicroUI.MicroUI.errorLog(e);
		}

		return microUIFonts;
	}

	private static int getStyles(int all, int mask, int shift) {
		return (all & mask) >> shift;
	}

	private static int getStyleFlag(int styles, int expectedStyleMask, int flag) {
		return (styles & expectedStyleMask) != 0 ? flag : 0;
	}

	/**
	 * Check the error returned during a font use: drawText etc. Does not check the
	 * font loading error.
	 *
	 * @param ret
	 *            the potential error or the original return code
	 * @return the original return code when there is no error
	 * @throws MicroUIException
	 *             when an error is detected
	 */
	private static int checkFontRuntimeError(int ret) {

		/*
		 * WARNING this code is duplicated in microui-extended
		 */

		if (ret == NSystemDisplay.INVALID_FILE) {
			// error when reading the font
			throw new MicroUIException(MicroUIException.RESOURCE_INVALID_FILE);
		} else if (ret == NSystemDisplay.FILE_CLOSED) {
			// font (master font data) has been closed
			throw new MicroUIException(MicroUIException.RESOURCE_CLOSED);
		}
		return ret;
	}

	/**
	 * Check the error returned during a font loading: addFont, getIdentifiers etc.
	 * @param ret
	 *
	 * @return
	 */
	/* default */ static int checkFontLoadingError(int ret) throws IOException {
		if (ret == NSystemDisplay.INVALID_PATH) {
			throw new MicroUIException(MicroUIException.RESOURCE_INVALID_PATH);
		} else if (ret == NSystemDisplay.INVALID_FILE) {
			throw new MicroUIException(MicroUIException.RESOURCE_INVALID_FILE);
		}
		return ret;
	}

	private int getDrawingWidth(GraphicsContext gc, char[] data, int offset, int length, int x) {
		// Width is useless in nominal case:
		// - ellipsis is false
		// - x start coordinate is in the clip
		// In this case, no need to retrieve width: it is never used by native algorithm
		//
		// Exceptions:
		// - If ellipsis is true, native algorithm requires the string width to
		// know when drawing the ellipsis.
		// - If x start coordinate is out of the clip on the left, native
		// algorithm requires the string width to know if the full string is
		// outside the clip on the left. In this case, no need to draw the
		// string.
		return gc.getEllipsisWidth() == 0 && x >= gc.getClipX() /* isOutsideClipOnTheLeft */ ? 0
				: getWidth(data, offset, length, null);
	}

	/* default */ void drawChars(GraphicsContext gc, char[] data, int offset, int length, int x, int y) {
		drawChars(gc, data, offset, length, x, y, null, getDrawingWidth(gc, data, offset, length, x));
	}

	/* default */ void drawChars(GraphicsContext gc, char[] data, int offset, int length, int x, int y, float factorX,
			float factorY) {
		drawChars(gc, data, offset, length, x, y, null, getDrawingWidth(gc, data, offset, length, x), factorX, factorY);
	}

	/**
	 * not in API
	 *
	 * @param width
	 *            size of string in pixel. 0 is a valid value
	 * @see #getDrawingWidth(GraphicsContext, char[], int, int, int)
	 */
	/* default */ void drawChars(GraphicsContext gc, char[] data, int offset, int length, int x, int y, int[] indexes,
			int width) {
		Font.checkFontRuntimeError(
				NSystemDisplay.drawChars(gc.getSNIContext(), getFontIdentifier(), x + gc.getTranslateX(),
						y + gc.getTranslateY(), data, offset, length, width, indexes, mallocExternalFontHeap()));
	}

	/**
	 * not in API
	 *
	 * @param width
	 *            size of string in pixel without x ratio. 0 is a valid value
	 * @see #getDrawingWidth(GraphicsContext, char[], int, int, int)
	 */
	/* default */ void drawChars(GraphicsContext gc, char[] data, int offset, int length, int x, int y, int[] indexes,
			int width, float factorX, float factorY) {
		Font.checkFontRuntimeError(NSystemDisplay.drawDeformedChars(gc.getSNIContext(), getFontIdentifier(),
				x + gc.getTranslateX(), y + gc.getTranslateY(), data, offset, length, width, factorX, factorY, indexes,
				mallocExternalFontHeap()));
	}

	/**
	 * Invariant data for a font file (hold by several {@link #Font})
	 */
	private static class FontsData implements Resource {

		/*
		 * Offsets of data filled by native side during font loading
		 */

		/**
		 * Native ID: address, external resources ID or array index (sim only)
		 */
		private static final int LOAD_OFFSET_ID = 0;

		/**
		 * external | monospace | styles
		 */
		private static final int LOAD_OFFSET_FLAGS = 4;

		/**
		 * size of identifiers array
		 */
		private static final int LOAD_OFFSET_ARRAY_ID_SIZE = 6;

		/**
		 * size of descriptor
		 */
		private static final int LOAD_OFFSET_ARRAY_DESC_SIZE = 7;

		private static final int LOAD_OFFSET_HEIGHT = 8;
		private static final int LOAD_OFFSET_BASELINE = 10;

		/**
		 * When font is external, this size is the byte array to load in java heap to
		 * load bigger character in the font. When font is internal, this size is 0.
		 */
		/* default */ static final int LOAD_OFFSET_EXTERNAL_HEAP_SIZE = 12;

		/**
		 * Size of array to allocate
		 */
		/* default */ static final int LOAD_OFFSETS_ARRAY_SIZE = 16;

		/**
		 * Font identifier when a font has been closed
		 */
		private static final int FONT_CLOSED_ID = -1;

		/**
		 * List of all fonts loaded at same time than current font. This object is
		 * unique for all fonts.
		 */
		private final String list;

		/**
		 * Start index of path of this font in the previous list
		 */
		private final char pathStartIndexInList;

		/**
		 * Start index of filename of this font in the previous list
		 */
		private final char filenameStartIndexInList;

		/**
		 * End index of filename of this font in the previous list
		 */
		private final char filenameEndIndexInList;

		private final int[] identifiers;

		/**
		 * Native font identifier. Some fields are extracted (inlined) because there are
		 * often used.
		 */
		private final byte[] nativeData;

		/**
		 * Font unique identifier. On sim, it is an unique index in a table. On emb
		 * (internal resource), this is the address of the RAW file. On emb (external
		 * resource, this is the external resource identifier. This identifier equals -1
		 * when the RAW font has been closed.
		 */
		private int identifier;

		private final char height;
		private final char baseline;

		/**
		 * When the font is an external resource, this size equals the heap size the
		 * application has to allocate to use the font. When the font is internal, this
		 * value is null (which means the font is internal).
		 */
		private final int externalHeapSize;

		private FontsData(String path, byte[] fontData, final String list) throws IOException {
			this.nativeData = fontData;
			this.list = list;
			this.pathStartIndexInList = (char) list.indexOf(path);
			String filename = path.substring(path.lastIndexOf('/') + 1);
			this.filenameStartIndexInList = (char) (list.indexOf(filename, this.pathStartIndexInList));
			this.filenameEndIndexInList = (char) (this.filenameStartIndexInList + filename.lastIndexOf('.'));
			this.identifier = Tools.getInt(fontData, LOAD_OFFSET_ID);
			this.height = Tools.getChar(fontData, LOAD_OFFSET_HEIGHT);
			this.baseline = Tools.getChar(fontData, LOAD_OFFSET_BASELINE);
			this.externalHeapSize = Tools.getInt(fontData, LOAD_OFFSET_EXTERNAL_HEAP_SIZE);

			// get identifiers array
			int l = Tools.getUnsignedByte(fontData, LOAD_OFFSET_ARRAY_ID_SIZE);
			if (l > 0) {
				int[] array = new int[l];
				Font.checkFontLoadingError(NSystemDisplay.getFontIdentifiers(this.identifier, array, isExternalFont()));
				this.identifiers = array;
			} else {
				this.identifiers = new int[] { Font.LATIN };
			}
		}

		private String getFilename() {
			return this.list.substring(this.filenameStartIndexInList, this.filenameEndIndexInList);
		}

		private boolean isSamePath(String path) {
			return this.list.regionMatches(this.pathStartIndexInList, path, 0, path.length);
		}

		/**
		 * An external font requires a heap to allocate character data. This heap is
		 * null when the font is not external (native uses this null array to
		 * determinate if given font is internal or external)
		 */
		private byte[] mallocExternalFontHeap() {
			return this.externalHeapSize == 0 ? null : new byte[this.externalHeapSize];
		}

		private boolean isMonospace() {
			return isFlagSet(FONT_MASK_MONO);
		}

		private int getStyles() {
			return Tools.getChar(this.nativeData, LOAD_OFFSET_FLAGS);
		}

		/**
		 * font is an external font if and only if we have to allocate a buffer to load
		 * the external characters
		 */
		private boolean isExternalFont() {
			return this.externalHeapSize > 0;
		}

		@Override
		public Object getSource() {
			return this;
		}

		@Override
		public synchronized void reclaim() {
			if (this.identifier != FONT_CLOSED_ID) {
				NSystemDisplay.closeFont(this.identifier, isExternalFont());
				this.identifier = FONT_CLOSED_ID;
				Tools.setInt(this.nativeData, LOAD_OFFSET_ID, FONT_CLOSED_ID);
				ResourceManager rm = ResourceManager.getResourceManager();
				if (rm != null) {
					rm.resourceReclaimed(this);
				}
			}
		}

		private boolean isFlagSet(int flag) {
			return (Tools.getChar(this.nativeData, LOAD_OFFSET_FLAGS) & flag) == flag;
		}
	}
}
