/*
 * Java
 *
 * Copyright 2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.microui.display;

import com.is2t.tools.ArrayTools;

/**
 * microUI-API
 */
public class Painter {

	private Painter() {
		// static methods only
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 */
	public static void drawPixel(GraphicsContext gc, int x, int y) {
		PainterNatives.drawPixel(gc.getSNIContext(), x + gc.translateX, y + gc.translateY);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 */
	public static void drawHorizontalLine(GraphicsContext gc, int x, int y, int width) {
		// if width == 0, nothing to draw (test performed on native side)
		PainterNatives.drawHorizontalLine(gc.getSNIContext(), x + gc.translateX, y + gc.translateY, width);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 */
	public static void drawVerticalLine(GraphicsContext gc, int x, int y, int height) {
		// if height == 0, nothing to draw (test performed on native side)
		PainterNatives.drawVerticalLine(gc.getSNIContext(), x + gc.translateX, y + gc.translateY, height);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x1
	 *            microUI-API
	 * @param y1
	 *            microUI-API
	 * @param x2
	 *            microUI-API
	 * @param y2
	 *            microUI-API
	 */
	public static void drawLine(GraphicsContext gc, int x1, int y1, int x2, int y2) {
		PainterNatives.drawLine(gc.getSNIContext(), x1 + gc.translateX, y1 + gc.translateY, x2 + gc.translateX,
				y2 + gc.translateY);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 */
	public static void drawRectangle(GraphicsContext gc, int x, int y, int width, int height) {
		// if width == 0 || height == 0, nothing to draw (test performed on native side)
		PainterNatives.drawRectangle(gc.getSNIContext(), x + gc.translateX, y + gc.translateY, width, height);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 */
	public static void fillRectangle(GraphicsContext gc, int x, int y, int width, int height) {
		PainterNatives.fillRectangle(gc.getSNIContext(), x + gc.translateX, y + gc.translateY, width, height);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 */
	public static void fillGraphicsContext(GraphicsContext gc) {
		PainterNatives.fillRectangle(gc.getSNIContext(), 0, 0, gc.getMetadata().getWidth(),
				gc.getMetadata().getHeight());
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 * @param arcWidth
	 *            microUI-API
	 * @param arcHeight
	 *            microUI-API
	 */
	public static void drawRoundRectangle(GraphicsContext gc, int x, int y, int width, int height, int arcWidth,
			int arcHeight) {
		PainterNatives.drawRoundRectangle(gc.getSNIContext(), x + gc.translateX, y + gc.translateY, width, height,
				arcWidth, arcHeight);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 * @param arcWidth
	 *            microUI-API
	 * @param arcHeight
	 *            microUI-API
	 */
	public static void fillRoundRectangle(GraphicsContext gc, int x, int y, int width, int height, int arcWidth,
			int arcHeight) {
		PainterNatives.fillRoundRectangle(gc.getSNIContext(), x + gc.translateX, y + gc.translateY, width, height,
				arcWidth, arcHeight);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param diameter
	 *            microUI-API
	 * @param startAngle
	 *            microUI-API
	 * @param arcAngle
	 *            microUI-API
	 */
	public static void drawCircleArc(GraphicsContext gc, int x, int y, int diameter, float startAngle, float arcAngle) {
		PainterNatives.drawCircleArc(gc.getSNIContext(), gc.translateX + x, gc.translateY + y, diameter, startAngle,
				arcAngle);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 * @param startAngle
	 *            microUI-API
	 * @param arcAngle
	 *            microUI-API
	 */
	public static void drawEllipseArc(GraphicsContext gc, int x, int y, int width, int height, float startAngle,
			float arcAngle) {
		PainterNatives.drawEllipseArc(gc.getSNIContext(), gc.translateX + x, gc.translateY + y, width, height,
				startAngle, arcAngle);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param diameter
	 *            microUI-API
	 * @param startAngle
	 *            microUI-API
	 * @param arcAngle
	 *            microUI-API
	 */
	public static void fillCircleArc(GraphicsContext gc, int x, int y, int diameter, float startAngle, float arcAngle) {
		PainterNatives.fillCircleArc(gc.getSNIContext(), gc.translateX + x, gc.translateY + y, diameter, startAngle,
				arcAngle);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 * @param startAngle
	 *            microUI-API
	 * @param arcAngle
	 *            microUI-API
	 */
	public static void fillEllipseArc(GraphicsContext gc, int x, int y, int width, int height, float startAngle,
			float arcAngle) {
		PainterNatives.fillEllipseArc(gc.getSNIContext(), gc.translateX + x, gc.translateY + y, width, height,
				startAngle, arcAngle);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param diameter
	 *            microUI-API
	 */
	public static void drawCircle(GraphicsContext gc, int x, int y, int diameter) {
		PainterNatives.drawCircle(gc.getSNIContext(), gc.translateX + x, gc.translateY + y, diameter);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param diameter
	 *            microUI-API
	 */
	public static void fillCircle(GraphicsContext gc, int x, int y, int diameter) {
		PainterNatives.fillCircle(gc.getSNIContext(), gc.translateX + x, gc.translateY + y, diameter);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 */
	public static void drawEllipse(GraphicsContext gc, int x, int y, int width, int height) {
		PainterNatives.drawEllipse(gc.getSNIContext(), gc.translateX + x, gc.translateY + y, width, height);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 */
	public static void fillEllipse(GraphicsContext gc, int x, int y, int width, int height) {
		PainterNatives.fillEllipse(gc.getSNIContext(), gc.translateX + x, gc.translateY + y, width, height);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param xSrc
	 *            microUI-API
	 * @param ySrc
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 * @param xDest
	 *            microUI-API
	 * @param yDest
	 *            microUI-API
	 */
	public static void drawDisplayRegion(GraphicsContext gc, int xSrc, int ySrc, int width, int height, int xDest,
			int yDest) {
		drawRegion(gc, Display.getDisplay().getGraphicsContext().getMetadata(), xSrc, ySrc, width, height, xDest, yDest,
				GraphicsContext.OPAQUE);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param xSrc
	 *            microUI-API
	 * @param ySrc
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 * @param xDest
	 *            microUI-API
	 * @param yDest
	 *            microUI-API
	 * @param alpha
	 *            microUI-API
	 */
	public static void drawDisplayRegion(GraphicsContext gc, int xSrc, int ySrc, int width, int height, int xDest,
			int yDest, int alpha) {
		drawRegion(gc, Display.getDisplay().getGraphicsContext().getMetadata(), xSrc, ySrc, width, height, xDest, yDest,
				alpha);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param font
	 *            microUI-API
	 * @param str
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 */
	public static void drawString(GraphicsContext gc, Font font, String str, int x, int y) {
		font.drawChars(gc, str.chars, str.offset, str.length() /* can throw NPE (spec) */, x, y);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param font
	 *            microUI-API
	 * @param str
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param factorX
	 *            microUI-API
	 * @param factorY
	 *            microUI-API
	 */
	public static void drawString(GraphicsContext gc, Font font, String str, int x, int y, float factorX,
			float factorY) {
		font.drawChars(gc, str.chars, str.offset, str.length() /* can throw NPE (spec) */, x, y, factorX, factorY);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param font
	 *            microUI-API
	 * @param str
	 *            microUI-API
	 * @param offset
	 *            microUI-API
	 * @param len
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 */
	public static void drawSubstring(GraphicsContext gc, Font font, String str, int offset, int len, int x, int y) {
		checkSubstringBounds(str, offset, len);
		font.drawChars(gc, str.chars, str.offset + offset, len, x, y);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param font
	 *            microUI-API
	 * @param str
	 *            microUI-API
	 * @param offset
	 *            microUI-API
	 * @param len
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param factorX
	 *            microUI-API
	 * @param factorY
	 *            microUI-API
	 */
	public static void drawSubstring(GraphicsContext gc, Font font, String str, int offset, int len, int x, int y,
			float factorX, float factorY) {
		checkSubstringBounds(str, offset, len);
		font.drawChars(gc, str.chars, str.offset + offset, len, x, y, factorX, factorY);
	}

	private static void checkSubstringBounds(String str, int offset, int len) {
		try {
			// check bounds, throw ArrayIndexOutOfBoundsException if wrong
			// values; str.length() can throw NPE
			ArrayTools.checkArrayBounds(str.length(), offset, len);
		} catch (ArrayIndexOutOfBoundsException e) {
			throw new StringIndexOutOfBoundsException();
		}
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param font
	 *            microUI-API
	 * @param character
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 */
	public static void drawChar(GraphicsContext gc, Font font, char character, int x, int y) {
		font.drawChars(gc, new char[] { character }, 0, 1, x, y);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param font
	 *            microUI-API
	 * @param character
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param factorX
	 *            microUI-API
	 * @param factorY
	 *            microUI-API
	 */
	public static void drawChar(GraphicsContext gc, Font font, char character, int x, int y, float factorX,
			float factorY) {
		font.drawChars(gc, new char[] { character }, 0, 1, x, y, factorX, factorY);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param font
	 *            microUI-API
	 * @param data
	 *            microUI-API
	 * @param offset
	 *            microUI-API
	 * @param length
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 */
	public static void drawChars(GraphicsContext gc, Font font, char[] data, int offset, int length, int x, int y) {
		ArrayTools.checkBounds(data.length, offset, length);
		font.drawChars(gc, data, offset, length, x, y);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param font
	 *            microUI-API
	 * @param data
	 *            microUI-API
	 * @param offset
	 *            microUI-API
	 * @param length
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param factorX
	 *            microUI-API
	 * @param factorY
	 *            microUI-API
	 */
	public static void drawChars(GraphicsContext gc, Font font, char[] data, int offset, int length, int x, int y,
			float factorX, float factorY) {
		ArrayTools.checkBounds(data.length, offset, length);
		font.drawChars(gc, data, offset, length, x, y, factorX, factorY);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param c
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 */
	public static void drawRenderableChar(GraphicsContext gc, RenderableChar c, int x, int y) {
		// no specific optimization
		drawChar(gc, c.font, c.c, x, y);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param c
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param factorX
	 *            microUI-API
	 * @param factorY
	 *            microUI-API
	 */
	public static void drawRenderableChar(GraphicsContext gc, RenderableChar c, int x, int y, float factorX,
			float factorY) {
		// no specific optimization
		drawChar(gc, c.font, c.c, x, y, factorX, factorY);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param string
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 */
	public static void drawRenderableString(GraphicsContext gc, RenderableString string, int x, int y) {
		String s = string.string;
		string.font.drawChars(gc, s.chars, s.offset, s.length, x, y, string.indexes, string.width);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param string
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param factorX
	 *            microUI-API
	 * @param factorY
	 *            microUI-API
	 */
	public static void drawRenderableString(GraphicsContext gc, RenderableString string, int x, int y, float factorX,
			float factorY) {
		String s = string.string;
		string.font.drawChars(gc, s.chars, s.offset, s.length, x, y, string.indexes, string.width, factorX, factorY);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 */
	public static void drawImage(GraphicsContext gc, Image image, int x, int y) {
		drawRegion(gc, image.getImageMetadata(), 0, 0, image.getWidth(), image.getHeight(), x, y,
				GraphicsContext.OPAQUE);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param x
	 *            microUI-API
	 * @param y
	 *            microUI-API
	 * @param alpha
	 *            microUI-API
	 */
	public static void drawImage(GraphicsContext gc, Image image, int x, int y, int alpha) {
		drawRegion(gc, image.getImageMetadata(), 0, 0, image.getWidth(), image.getHeight(), x, y, alpha);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param xSrc
	 *            microUI-API
	 * @param ySrc
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 * @param xDest
	 *            microUI-API
	 * @param yDest
	 *            microUI-API
	 */
	public static void drawImageRegion(GraphicsContext gc, Image image, int xSrc, int ySrc, int width, int height,
			int xDest, int yDest) {
		drawRegion(gc, image.getImageMetadata(), xSrc, ySrc, width, height, xDest, yDest, GraphicsContext.OPAQUE);
	}

	/**
	 * microUI-API
	 *
	 * @param gc
	 *            microUI-API
	 * @param image
	 *            microUI-API
	 * @param xSrc
	 *            microUI-API
	 * @param ySrc
	 *            microUI-API
	 * @param width
	 *            microUI-API
	 * @param height
	 *            microUI-API
	 * @param xDest
	 *            microUI-API
	 * @param yDest
	 *            microUI-API
	 * @param alpha
	 *            microUI-API
	 */
	public static void drawImageRegion(GraphicsContext gc, Image image, int xSrc, int ySrc, int width, int height,
			int xDest, int yDest, int alpha) {
		drawRegion(gc, image.getImageMetadata(), xSrc, ySrc, width, height, xDest, yDest, alpha);
	}

	private static void drawRegion(GraphicsContext gc, SImageMetadata srcSd, int xSrc, int ySrc, int width, int height,
			int xDest, int yDest, int alpha) {

		if (alpha < GraphicsContext.TRANSPARENT || alpha > GraphicsContext.OPAQUE) {
			throw new IllegalArgumentException();
		}

		// compute destination area
		xDest += gc.translateX;
		yDest += gc.translateY;

		PainterNatives.drawImage(gc.getSNIContext(), srcSd.getGraphicalEngineData(), xSrc, ySrc, width, height, xDest,
				yDest, alpha);
	}
}
