/*
 * Java
 *
 * Copyright 2017-2020 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.microui.display;

/**
 * This class associates a string with a font. The string rendering is
 * accelerated because the couple string/font is fixed.
 */
public class RenderableString {

	/**
	 * This default index is used when the string indexes have not been computed
	 * yet.
	 */
	private static final int INVALID_INDEX = -1;

	/* default */ final String string;
	/* default */ final int[] indexes;
	/* default */ final Font font;
	/* default */ final int width;

	/**
	 * microUI-API
	 *
	 * @param string
	 *            microUI-API
	 * @param font
	 *            microUI-API
	 */
	public RenderableString(String string, Font font) {

		this.string = string;
		this.font = font;

		// create array of indexes; initializes it with invalid indexes in order
		// to force the native side to retrieve all characters indexes
		int l = string.length(); // NPE
		this.indexes = new int[l];
		for (int i = l; --i >= 0;) {
			this.indexes[i] = INVALID_INDEX;
		}

		// get width without ratio (this size will never change)
		final char[] data = this.string.chars;
		final int offset = this.string.offset;
		final int length = this.string.length;
		this.width = font.getWidth(data, offset, length, this.indexes); // NPE
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public String getString() {
		return this.string;
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public Font getFont() {
		return this.font;
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public int getWidth() {
		return this.width;
	}

	/**
	 * microUI-API
	 *
	 * @return microUI-API
	 */
	public int getHeight() {
		return this.font.getHeight();
	}
}
